@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-filetypes.c,v 1.54 2004/02/27 19:23:20 bozo Exp $
+  $Id: gsm-filetypes.c,v 1.74 2007/06/12 16:00:32 dforsi Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,20 +22,26 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000-2001 Chris Kemp, Marcin Wiacek
+  Copyright (C) 2000-2004 Pawel Kot
+  Copyright (C) 2001      Bartek Klepacz
+  Copyright (C) 2002      Markus Plail, Pavel Machek
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003      Tomi Ollila
 
   Functions to read and write common file types.
 
 */
 
+#include "config.h"
+#include "compat.h"
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include "config.h"
-#include "compat.h"
 #include "gnokii-internal.h"
 #include "gnokii.h"
 #include "gsm-filetypes.h"
@@ -52,7 +58,7 @@
 
 
 /* Function to convert scale field in to correct number. */
-static int ringtone_get_duration (char *num)
+static int ringtone_get_duration(char *num)
 {
 	int duration = 0;
 
@@ -80,7 +86,7 @@ static int ringtone_get_duration (char *num)
 }
 
 
-static int ringtone_get_scale (char *num)
+static int ringtone_get_scale(char *num)
 {
 	/* This may well need improving. */
 	int scale=0;
@@ -588,7 +594,7 @@ gn_error gn_file_bitmap_read(char *filename, gn_bmp *bitmap, gn_phone *info)
 		error = file_xpm_load(filename, bitmap);
 		break;
 #else
-		fprintf(stderr, "Sorry, gnokii was not compiled with XPM support.\n");
+		fprintf(stderr, _("Sorry, gnokii was not compiled with XPM support.\n"));
 		/* FALLTHRU */
 #endif
 	default:
@@ -631,7 +637,7 @@ gn_error file_xpm_load(char *filename, gn_bmp *bitmap)
 	bitmap->size = ((bitmap->width + 7) / 8) * bitmap->height;
 
 	if (bitmap->size > GN_BMP_MAX_SIZE) {
-		fprintf(stdout, "Bitmap too large\n");
+		fprintf(stderr, _("Bitmap too large\n"));
 		return GN_ERR_INVALIDSIZE;
 	}
 
@@ -1063,7 +1069,7 @@ int gn_file_text_save(char *filename, char *text, int mode)
 	if ((mode == 1) && (stat(filename, &buf) == 0)) {
 		fprintf(stdout, _("File %s exists.\n"), filename);
 		while (confirm < 0) {
-			fprintf(stderr, _("Overwrite? (yes/no) "));
+			fprintf(stdout, _("Overwrite? (yes/no) "));
 			gn_line_get(stdin, ans, 4);
 			if (!strcmp(ans, _("yes"))) confirm = 1;
 			else if (!strcmp(ans, _("no"))) confirm = 0;
@@ -1075,7 +1081,7 @@ int gn_file_text_save(char *filename, char *text, int mode)
 	else file = fopen(filename, "w");
 
 	if (!file) {
-		fprintf(stderr, _("Failed to write file %s\n"),  filename);
+		fprintf(stderr, _("Can't open file %s for writing!\n"),  filename);
 		return -1;
 	}
 	fprintf(file, "%s\n", text);
@@ -1378,100 +1384,269 @@ gn_error gn_file_bitmap_show(char *filename)
 
 	for (i = 0; i < bitmap.height; i++) {
 		for (j = 0; j < bitmap.width; j++) {
-			printf("%c", gn_bmp_point(&bitmap, j, i) ? '#' : ' ');
+			fprintf(stdout, "%c", gn_bmp_point(&bitmap, j, i) ? '#' : ' ');
 		}
-		printf("\n");
+		fprintf(stdout, "\n");
 	}
 
 	return GN_ERR_NONE;
 }
 
-#define MAX_INPUT_LINE_LEN 512
-API gn_error gn_file_phonebook_raw(gn_phonebook_entry *entry, char *oline)
+/* returns number of the characters before the next delimiter, including it */
+static int get_next_token(char *src, int delim)
 {
-	char *line = oline;
-	char *ptr;
-	char backline[MAX_INPUT_LINE_LEN];
+	int i, len = strlen(src);
+	int slash_state = 0;
 
+	for (i = 0; i < len; i++) {
+		switch (src[i]) {
+		case '\\':
+			if (slash_state) {
+				slash_state = 0;
+			} else {
+				slash_state = 1;
+			}
+			break;
+		case ';':
+			if (slash_state)
+				slash_state = 0;
+			else
+				return i + 1;
+			break;
+		default:
+			if (slash_state)
+				slash_state = 0;
+			break;
+		}
+	}
+	return i + 1;
+}
+
+#define BUG(x) \
+	do { \
+		if (x) { \
+			return GN_ERR_WRONGDATAFORMAT; \
+		} \
+	} while (0)
+
+#define MAX_INPUT_LINE_LEN 512
+
+#define GET_NEXT_TOKEN()	o = get_next_token(line + offset, ';')
+#define STORE_TOKEN(a)		strip_slashes(a, line + offset, sizeof(a) - 1, o - 1)
+
+inline int local_atoi(char *str, int len)
+{
+	int retval;
+	char *aux = strndup(str, len);
+	retval = atoi(aux);
+	free(aux);
+	return retval;
+}
+
+GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char *line)
+{
+	char backline[MAX_INPUT_LINE_LEN];
+	char memory_type_char[3];
+	char number[GN_PHONEBOOK_NUMBER_MAX_LENGTH];
+	int length, o, offset = 0;
+	gn_error error = GN_ERR_NONE;
+
+	memset(entry, 0, sizeof(gn_phonebook_entry));
+
+	length = strlen(line);
 	strcpy(backline, line);
 	entry->empty = true;
+	memory_type_char[2] = 0;
 
-	ptr = strsep(&line, ";");
-	if (ptr) strncpy(entry->name, ptr, sizeof(entry->name) - 1);
+	GET_NEXT_TOKEN();
+	switch (o) {
+	case 0:
+		return GN_ERR_WRONGDATAFORMAT;
+	case 1:
+		/* empty name: this is a request to delete the entry */
+		break;
+	default:
+		break;
+	}
+	STORE_TOKEN(entry->name);
+	offset += o;
 
-	ptr = strsep(&line, ";");
-	if (ptr) strncpy(entry->number, ptr, sizeof(entry->number) - 1);
+	BUG(offset >= length);
 
-	ptr = strsep(&line, ";");
+	GET_NEXT_TOKEN();
+	switch (o) {
+	case 0:
+		return GN_ERR_WRONGDATAFORMAT;
+	default:
+		break;
+	}
+	STORE_TOKEN(entry->number);
+	offset += o;
 
-	if (!ptr) {
-		fprintf(stderr, _("Format problem on line [%s]\n"), backline);
-		line = oline;
+	BUG(offset >= length);
+
+	GET_NEXT_TOKEN();
+	switch (o) {
+	case 3:
+		break;
+	default:
 		return GN_ERR_WRONGDATAFORMAT;
 	}
+	STORE_TOKEN(memory_type_char);
+	offset += o;
 
-	if (!strncmp(ptr, "ME", 2)) {
-		entry->memory_type = GN_MT_ME;
-	} else {
-		if (!strncmp(ptr, "SM", 2)) {
-			entry->memory_type = GN_MT_SM;
-		} else {
-			fprintf(stderr, _("Format problem on line [%s]\n"), backline);
-			return GN_ERR_WRONGDATAFORMAT;
-		}
+	BUG(offset >= length);
+
+	entry->memory_type = gn_str2memory_type(memory_type_char);
+	/* We can store addressbook entries only in ME or SM (or ON on some models) */
+	if (entry->memory_type != GN_MT_ME &&
+	    entry->memory_type != GN_MT_SM &&
+	    entry->memory_type != GN_MT_ON) {
+		return GN_ERR_INVALIDMEMORYTYPE;
 	}
 
-	ptr = strsep(&line, ";");
-	if (ptr) entry->location = atoi(ptr);
-	else entry->location = 0;
+	BUG(offset >= length);
 
-	ptr = strsep(&line, ";");
-	if (ptr) entry->caller_group = atoi(ptr);
-	else entry->caller_group = 0;
-
-	if (!ptr) {
-		fprintf(stderr, _("Format problem on line [%s]\n"), backline);
+	memset(number, 0, sizeof(number));
+	GET_NEXT_TOKEN();
+	STORE_TOKEN(number);
+	switch (o) {
+	case 0:
 		return GN_ERR_WRONGDATAFORMAT;
+	case 1:
+		entry->location = 0;
+		break;
+	default:
+		entry->location = atoi(number);
+		break;
 	}
+	offset += o;
 
+	BUG(offset >= length);
+
+	memset(number, 0, sizeof(number));
+	GET_NEXT_TOKEN();
+	STORE_TOKEN(number);
+	switch (o) {
+	case 0:
+		return GN_ERR_WRONGDATAFORMAT;
+	case 1:
+		entry->caller_group = 0;
+		break;
+	default:
+		entry->caller_group = atoi(number);
+		break;
+	}
+	offset += o;
+	
 	entry->empty = false;
 
-	for (entry->subentries_count = 0; ; entry->subentries_count++) {
-		ptr = strsep(&line, ";");
-
-		if (ptr && *ptr != 0)
-			entry->subentries[entry->subentries_count].entry_type = atoi(ptr);
-		else
+	for (entry->subentries_count = 0; offset < length; entry->subentries_count++) {
+		memset(number, 0, sizeof(number));
+		GET_NEXT_TOKEN();
+		STORE_TOKEN(number);
+		switch (o) {
+		case 0:
+			fprintf(stderr, _("Formatting error: unknown error while reading subentry type\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		case 1:
+			fprintf(stderr, _("Formatting error: empty entry type\n"));
+			entry->subentries[entry->subentries_count].entry_type = 0;
 			break;
+		default:
+			entry->subentries[entry->subentries_count].entry_type = atoi(number);
+			break;
+		}
+		offset += o;
 
-		ptr = strsep(&line, ";");
-		if (ptr) entry->subentries[entry->subentries_count].number_type = atoi(ptr);
-
-		/* Phone Numbers need to have a number type. */
-		if (!ptr && entry->subentries[entry->subentries_count].entry_type == GN_PHONEBOOK_ENTRY_Number) {
-			fprintf(stderr, _("Missing phone number type on line %d"
-					  " entry [%s]\n"), entry->subentries_count, backline);
-			entry->subentries_count--;
+		if (offset > length) {
+			fprintf(stderr, _("Formatting error: subentry has only entry type field\n"));
 			break;
 		}
 
-		ptr = strsep(&line, ";");
-		if (ptr) entry->subentries[entry->subentries_count].id = atoi(ptr);
-
-		ptr = strsep(&line, ";");
-
-		/* 0x13 Date Type; it is only for Dailed Numbers, etc.
-		   we don't store to this memories so it's an error to use it. */
-		if (!ptr || entry->subentries[entry->subentries_count].entry_type == GN_PHONEBOOK_ENTRY_Date) {
-			fprintf(stderr, _("There is no phone number on line [%s] entry %d\n"),
-				backline, entry->subentries_count);
-			entry->subentries_count--;
+		memset(number, 0, sizeof(number));
+		GET_NEXT_TOKEN();
+		STORE_TOKEN(number);
+		switch (o) {
+		case 0:
+			fprintf(stderr, _("Formatting error: unknown error while reading subentry number type\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		case 1:
+			fprintf(stderr, _("Formatting error: empty number type\n"));
+			entry->subentries[entry->subentries_count].number_type = 0;
+			/* Number type is required with Number entry type */
+			if (entry->subentries[entry->subentries_count].entry_type == GN_PHONEBOOK_ENTRY_Number) {
+				error = GN_ERR_WRONGDATAFORMAT;
+				goto endloop;
+			}
 			break;
-		} else
-			strncpy(entry->subentries[entry->subentries_count].data.number, ptr, sizeof(entry->subentries[entry->subentries_count].data.number) - 1);
+		default:
+			entry->subentries[entry->subentries_count].number_type = atoi(number);
+			break;
+		}
+		offset += o;
+
+		if (offset > length) {
+			fprintf(stderr, _("Formatting error: subentry has only entry and number type fields\n"));
+			break;
+		}
+
+		memset(number, 0, sizeof(number));
+		GET_NEXT_TOKEN();
+		STORE_TOKEN(number);
+		switch (o) {
+		case 0:
+			fprintf(stderr, _("Formatting error: unknown error while reading subentry id\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		case 1:
+			fprintf(stderr, _("Formatting error: empty id\n"));
+			entry->subentries[entry->subentries_count].id = 0;
+			break;
+		default:
+			entry->subentries[entry->subentries_count].id = atoi(number);
+			break;
+		}
+		offset += o;
+
+		if (offset > length) {
+			fprintf(stderr, _("Formatting error: subentry has only entry and number type fields\n"));
+			break;
+		}
+
+		GET_NEXT_TOKEN();
+		switch (entry->subentries[entry->subentries_count].entry_type) {
+			case GN_PHONEBOOK_ENTRY_Date:
+			case GN_PHONEBOOK_ENTRY_Birthday:
+				entry->subentries[entry->subentries_count].data.date.year   = local_atoi(line + offset, 4);
+				entry->subentries[entry->subentries_count].data.date.month  = local_atoi(line + offset + 4, 2);
+				entry->subentries[entry->subentries_count].data.date.day    = local_atoi(line + offset + 6, 2);
+				entry->subentries[entry->subentries_count].data.date.hour   = local_atoi(line + offset + 8, 2);
+				entry->subentries[entry->subentries_count].data.date.minute = local_atoi(line + offset + 10, 2);
+				entry->subentries[entry->subentries_count].data.date.second = local_atoi(line + offset + 12, 2);
+				break;
+			default:
+				STORE_TOKEN(entry->subentries[entry->subentries_count].data.number);
+				break;
+			}
+		switch (o) {
+		case 0:
+			fprintf(stderr, _("Formatting error: unknown error while reading subentry contents\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		case 1:
+			fprintf(stderr, _("Formatting error: empty subentry contents\n"));
+			break;
+		default:
+			break;
+		}
+		offset += o;
 	}
 
-	/* This is to send other exports (like from 6110) to 7110 */
+endloop:
+	/* Fake subentry: this is to send other exports (like from 6110) to 7110 */
 	if (!entry->subentries_count) {
 		entry->subentries[entry->subentries_count].entry_type   = GN_PHONEBOOK_ENTRY_Number;
 		entry->subentries[entry->subentries_count].number_type  = GN_PHONEBOOK_NUMBER_General;
@@ -1479,6 +1654,77 @@ API gn_error gn_file_phonebook_raw(gn_phonebook_entry *entry, char *oline)
 		strcpy(entry->subentries[entry->subentries_count].data.number, entry->number);
 		entry->subentries_count = 1;
 	}
-	return GN_ERR_NONE;
+	return error;
 }
 
+GNOKII_API gn_error gn_file_phonebook_raw_write(FILE *f, gn_phonebook_entry *entry, char *memory_type_string)
+{
+	char escaped_name[2 * GN_PHONEBOOK_NAME_MAX_LENGTH];
+	int i;
+
+	add_slashes(escaped_name, entry->name, sizeof(escaped_name), strlen(entry->name));
+	fprintf(f, "%s;%s;%s;%d;%d", escaped_name,
+		entry->number, memory_type_string,
+		entry->location, entry->caller_group);
+	if (entry->person.has_person) {
+		if (entry->person.honorific_prefixes[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_FormalName,
+				entry->person.honorific_prefixes);
+		if (entry->person.given_name[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_FirstName,
+				entry->person.given_name);
+		if (entry->person.family_name[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_LastName,
+				entry->person.family_name);
+	}
+	if (entry->address.has_address) {
+		if (entry->address.post_office_box[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Postal,
+				entry->address.post_office_box);
+		if (entry->address.extended_address[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_ExtendedAddress,
+				entry->address.extended_address);
+		if (entry->address.street[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Street,
+				entry->address.street);
+		if (entry->address.city[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_City,
+				entry->address.city);
+		if (entry->address.state_province[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_StateProvince,
+				entry->address.state_province);
+		if (entry->address.zipcode[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_ZipCode,
+				entry->address.zipcode);
+		if (entry->address.country[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Country,
+				entry->address.country);
+	}
+
+	for (i = 0; i < entry->subentries_count; i++) {
+		switch (entry->subentries[i].entry_type) {
+		case GN_PHONEBOOK_ENTRY_Birthday:
+		case GN_PHONEBOOK_ENTRY_Date:
+			fprintf(f, ";%d;0;%d;%04u%02u%02u%02u%02u%02u", entry->subentries[i].entry_type, entry->subentries[i].id,
+				entry->subentries[i].data.date.year, entry->subentries[i].data.date.month, entry->subentries[i].data.date.day,
+				entry->subentries[i].data.date.hour, entry->subentries[i].data.date.minute, entry->subentries[i].data.date.second);
+			break;
+		default:
+			add_slashes(escaped_name, entry->subentries[i].data.number, sizeof(escaped_name), strlen(entry->subentries[i].data.number));
+			fprintf(f, ";%d;%d;%d;%s",
+				entry->subentries[i].entry_type,
+				entry->subentries[i].number_type,
+				entry->subentries[i].id,
+				escaped_name);
+			break;
+		}
+	}
+	if ((entry->memory_type == GN_MT_MC ||
+		entry->memory_type == GN_MT_DC ||
+		entry->memory_type == GN_MT_RC) &&
+		entry->date.day != 0)
+		fprintf(f, ";%d;0;0;%04u%02u%02u%02u%02u%02u", GN_PHONEBOOK_ENTRY_Date,
+			entry->date.year, entry->date.month, entry->date.day, entry->date.hour, entry->date.minute, entry->date.second);
+	fprintf(f, "\n");
+	return GN_ERR_NONE;
+}
