@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-sms.c,v 1.151 2006/08/08 20:06:29 pkot Exp $
+  $Id: gsm-sms.c,v 1.159 2007/01/06 22:53:21 pkot Exp $
 
   G N O K I I
 
@@ -135,13 +135,13 @@ GNOKII_API void gn_sms_default_deliver(gn_sms *sms)
  * sms_timestamp_print - formats date and time in the human readable manner
  * @number: binary coded date and time
  *
- * The function converts binary date time into a easily readable form.
+ * The function converts binary date time into an easily readable form.
  * It is Y2K compliant.
  */
 static char *sms_timestamp_print(u8 *number)
 {
 #ifdef DEBUG
-#define LOCAL_DATETIME_MAX_LENGTH 23
+#define LOCAL_DATETIME_MAX_LENGTH 26
 	static char buffer[LOCAL_DATETIME_MAX_LENGTH];
 	char buf[5];
 	int i;
@@ -409,7 +409,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
  * @udhlen: udh length 
  * @dcs: data coding scheme
  *
- * This function decodes either 7bit or 8bit os Unicode text to the
+ * This function decodes either 7bit or 8bit or Unicode text to the
  * readable text format according to the locale set.
  */
 static gn_error sms_data_decode(unsigned char *message, unsigned char *output, unsigned int length,
@@ -437,11 +437,11 @@ static gn_error sms_data_decode(unsigned char *message, unsigned char *output, u
 }
 
 /**
- * sms_udh_decode - interprete the User Data Header contents
+ * sms_udh_decode - interprets the User Data Header contents
  * @message: received UDH
  * @sms: SMS structure to fill in
  *
- * At this stage we already need to know thet SMS has UDH.
+ * At this stage we already need to know that SMS has UDH.
  * This function decodes UDH as described in:
  *   - GSM 03.40 version 6.1.0 Release 1997, section 9.2.3.24
  *   - Smart Messaging Specification, Revision 1.0.0, September 15, 1997
@@ -465,6 +465,13 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 			udh->udh[nr].u.concatenated_short_message.maximum_number   = message[pos + 3];
 			udh->udh[nr].u.concatenated_short_message.current_number   = message[pos + 4];
 			break;
+		case 0x08: /* Concatenated short messages, 16-bit reference number */
+			dprintf("Concatenated messages, 16-bit reference number\n");
+			udh->udh[nr].type = GN_SMS_UDH_ConcatenatedMessages;
+			udh->udh[nr].u.concatenated_short_message.reference_number = 256 * message[pos + 2] + message[pos + 3];
+			udh->udh[nr].u.concatenated_short_message.maximum_number   = message[pos + 4];
+			udh->udh[nr].u.concatenated_short_message.current_number   = message[pos + 5];
+			break;
 		case 0x01: /* Special SMS Message Indication */
 			switch (message[pos + 2] & 0x03) {
 			case 0x00:
@@ -487,7 +494,7 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 			udh->udh[nr].u.special_sms_message_indication.store = (message[pos + 2] & 0x80) >> 7;
 			udh->udh[nr].u.special_sms_message_indication.message_count = message[pos + 3];
 			break;
-		case 0x05: /* Application port addression scheme, 16 bit address */
+		case 0x05: /* Application port addressing scheme, 16 bit address */
 			switch (((0x00ff & message[pos + 2]) << 8) | (0x00ff & message[pos + 3])) {
 			case 0x1581:
 				dprintf("Ringtone\n");
@@ -531,7 +538,7 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 				break;
 			}
 			break;
-		case 0x04: /* Application port addression scheme, 8 bit address */
+		case 0x04: /* Application port addressing scheme, 8 bit address */
 		case 0x06: /* SMSC Control Parameters */
 		case 0x07: /* UDH Source Indicator */
 		default:
@@ -552,13 +559,13 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 }
 
 /**
- * sms_header_decode - Doecodes PDU SMS header
+ * sms_header_decode - Decodes PDU SMS header
  * @rawsms:
  * @sms:
  * @udh:
  *
  * This function parses received SMS header information and stores
- * them in hihger level SMS struct. It also checks for the UDH and when
+ * them in higher level SMS struct. It also checks for the UDH and when
  * it's found calls the function to extract the UDH.
  */
 static gn_error sms_header_decode(gn_sms_raw *rawsms, gn_sms *sms, gn_sms_udh *udh)
@@ -684,12 +691,11 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 		break;
 	/* Plain text message */
 	default:
-		size = rawsms->length - sms->udh.length;
 		sms_data_decode(rawsms->user_data + sms->udh.length,        /* Skip the UDH */
 				(unsigned char *)&sms->user_data[0].u.text, /* With a plain text message we have only 1 part */
-				rawsms->length,                            /* Length of the decoded text */
-				size,                                      /* Length of the encoded text (in full octets) without UDH */
-				sms->udh.length,                          /* To skip the certain number of bits when unpacking 7bit message */
+				rawsms->length,                             /* Length of the decoded text */
+				rawsms->user_data_length,                   /* Length of the encoded text (in full octets) without UDH */
+				sms->udh.length,                            /* To skip the certain number of bits when unpacking 7bit message */
 				sms->dcs);
 		break;
 	}
@@ -713,12 +719,12 @@ GNOKII_API gn_error gn_sms_parse(gn_data *data)
 }
 
 /**
- * gn_sms_request - High-level function for the expicit SMS reading from the phone
+ * gn_sms_request - High-level function for the explicit SMS reading from the phone
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
  * This is the function for explicit requesting the SMS from the
- * phone driver. Not that raw_sms field in the gn_data structure must
+ * phone driver. Note that raw_sms field in the gn_data structure must
  * be initialized
  */
 GNOKII_API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
@@ -732,7 +738,7 @@ GNOKII_API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
- * This function is the frotnend for reading SMS. Note that SMS field
+ * This function is the frontend for reading SMS. Note that SMS field
  * in the gn_data structure must be initialized.
  */
 GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
@@ -752,7 +758,7 @@ GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
 }
 
 /**
- * gn_sms_delete - High-level function for delting SMS
+ * gn_sms_delete - High-level function for deleting SMS
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
@@ -890,7 +896,7 @@ static gn_error sms_verify_status(gn_data *data)
 				(data->message_list[i][data->sms_folder->folder_id]->status == GN_SMS_FLD_NotReadHandled)) {
 			for (j = 0; j < data->sms_folder->number; j++) {
 				if (data->message_list[i][data->sms_folder->folder_id]->location == data->sms_folder->locations[j]) {
-					/* We have a found a formerly unread message which has been read in the meantime */
+					/* We have found a formerly unread message which has been read in the meantime */
 					dprintf("Found a formerly unread message which has been read in the meantime: loc: %i\n",
 							data->message_list[i][data->sms_folder->folder_id]->location);
 					data->message_list[i][data->sms_folder->folder_id]->status = GN_SMS_FLD_Changed;
@@ -1051,7 +1057,7 @@ static gn_error sms_concat_header_encode(gn_sms_raw *rawsms, int curr, int total
  * @clen: coded data length
  *
  * This function does the phone frame encoding basing on the given SMS
- * structure. This function is capable to create only one frame at a time.
+ * structure. This function is capable of creating only one frame at a time.
  */
 static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 {
@@ -1138,26 +1144,28 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 			unsigned int length, udh_length, offset = rawsms->user_data_length;
 
 			length = strlen(sms->user_data[i].u.text);
+			if (sms->udh.length)
+				udh_length = sms->udh.length + 1;
+			else
+				udh_length = 0;
 			switch (al) {
 			case GN_SMS_DCS_DefaultAlphabet:
-				if (sms->udh.length) udh_length = sms->udh.length+1;
-				else udh_length = 0;
 				size = char_7bit_pack((7 - (udh_length % 7)) % 7,
 						      sms->user_data[i].u.text,
 						      rawsms->user_data + offset,
 						      &length);
-				rawsms->length = length;
+				rawsms->length = length + (udh_length * 8 + 6) / 7;
 				rawsms->user_data_length = size + offset;
 				break;
 			case GN_SMS_DCS_8bit:
 				rawsms->dcs |= 0xf4;
 				memcpy(rawsms->user_data + offset, sms->user_data[i].u.text, sms->user_data[i].u.text[0]);
-				rawsms->user_data_length = rawsms->length = length + offset;
+				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			case GN_SMS_DCS_UCS2:
 				rawsms->dcs |= 0x08;
 				length = char_unicode_encode(rawsms->user_data + offset, sms->user_data[i].u.text, length);
-				rawsms->user_data_length = rawsms->length = length + offset;
+				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			default:
 				return GN_ERR_WRONGDATAFORMAT;
@@ -1243,7 +1251,7 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 	}
 	/* Encoding the header */
 	rawsms->report = sms->delivery_report;
-	rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
+		rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
 	rawsms->validity_indicator = GN_SMS_VP_RelativeFormat;
 	rawsms->validity[0] = 0xa9;
 
@@ -1333,11 +1341,15 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 	gn_sms_user_data ud[GN_SMS_PART_MAX_NUMBER];
 	gn_error error = GN_ERR_NONE;
 
+	/* count -- number of SMS to be sent
+	 * total -- total number of octets to be sent
+	 */
 	sms = *data->sms;
 
 	/* If there's no concat header we need to add one */
 	for (i = 0; i < data->sms->number; i++)
-		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages) isConcat = i;
+		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
+			isConcat = i;
 
 	if (isConcat == -1)
 		isConcat = gn_sms_udh_add(data->sms, GN_SMS_UDH_ConcatenatedMessages);
@@ -1376,9 +1388,11 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 		switch (data->sms->dcs.u.general.alphabet) {
 		case GN_SMS_DCS_DefaultAlphabet:
 			start += copied;
-			copied = (ud[0].length - start) % ((MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7);
-			if (copied == 0)
-				copied = ((MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7);
+			if (ud[0].length - start >= (MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7) {
+				copied = (MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7;
+			} else {
+				copied = (ud[0].length - start) % ((MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7);
+			}
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
 			data->sms->user_data[0].length = copied;
@@ -1387,15 +1401,18 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 		case GN_SMS_DCS_UCS2:
 			start += copied;
 			copied = (ud[0].length - start) % ((MAX_SMS_PART - (data->sms->udh.length+1)));
-			if (copied == 0)
+			/* This is maximum size of what we can send */
+			if (copied == 0) {
 				copied = (MAX_SMS_PART - (data->sms->udh.length+1));
+			}
 			if (copied > (MAX_SMS_PART - (data->sms->udh.length+1)) / 2) {
 				copied = copied / 2;
 				if (copied < (MAX_SMS_PART - (data->sms->udh.length+1)) / 2)
 					copied = (MAX_SMS_PART - (data->sms->udh.length+1)) / 2;
-
 			}
-			dprintf("%d\n", copied);
+			if (i < count - 1 && copied % 2) {
+				copied--;
+			}
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
 			data->sms->user_data[0].length = copied;
