@@ -1,10 +1,10 @@
 /*
 
-  $Id: fbus-phonet.c,v 1.36 2004/06/27 20:24:14 pkot Exp $
+  $Id: fbus-phonet.c,v 1.42 2006/04/29 11:51:08 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2000 Hugh Blemings & Pavel Jank ml.
-  Copyright (C) 2000 Chris Kemp
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000-2001 Chris Kemp
+  Copyright (C) 2001-2004 Pawel Kot
+  Copyright (C) 2001      Manfred Jonsson, Martin Jancar
+  Copyright (C) 2002      Ladis Michl
+  Copyright (C) 2002-2003 BORBELY Zoltan
 
   This file provides an API for accessing functions via fbus over irda.
   See README for more details on supported mobile phones.
@@ -52,7 +56,7 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 static gn_error phonet_send_message(unsigned int messagesize, unsigned char messagetype, unsigned char *message, struct gn_statemachine *state);
 
 
-#define FBUSINST(s) ((phonet_incoming_message *)((s)->link.link_instance))
+#define FBUSINST(s) (*((phonet_incoming_message **)(&(s)->link.link_instance)))
 
 #define FBUS_PHONET_BLUETOOTH_INITSEQ 0xd0, 0x00, 0x01
 
@@ -117,7 +121,8 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 	switch (i->state) {
 	case FBUS_RX_Sync:
 		if (rx_byte == FBUS_PHONET_FRAME_ID ||
-		    rx_byte == FBUS_PHONET_BLUETOOTH_FRAME_ID) {
+		    rx_byte == FBUS_PHONET_BLUETOOTH_FRAME_ID ||
+		    rx_byte == FBUS_PHONET_DKU2_FRAME_ID) {
 			i->state = FBUS_RX_GetDestination;
 		}
 		break;
@@ -128,9 +133,10 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		i->state = FBUS_RX_GetSource;
 
 		if (rx_byte != FBUS_DEVICE_PC &&
+		    rx_byte != FBUS_PHONET_DKU2_DEVICE_PC &&
 		    rx_byte != FBUS_PHONET_BLUETOOTH_DEVICE_PC) {
 			i->state = FBUS_RX_Sync;
-			dprintf("The fbus stream is out of sync - expected 0x0c, got 0x%2x\n", rx_byte);
+			dprintf("The fbus stream is out of sync - expected 0x0c, got 0x%02x\n", rx_byte);
 		}
 		break;
 
@@ -141,7 +147,7 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 
 		if (rx_byte != FBUS_DEVICE_PHONE) {
 			i->state = FBUS_RX_Sync;
-			dprintf("The fbus stream is out of sync - expected 0x00, got 0x%2x\n", rx_byte);
+			dprintf("The fbus stream is out of sync - expected 0x00, got 0x%02x\n", rx_byte);
 		}
 
 		break;
@@ -164,7 +170,7 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 
 		i->message_length = i->message_length + rx_byte;
 		i->state = FBUS_RX_GetMessage;
-		i->buffer_count=0;
+		i->buffer_count = 0;
 
 		break;
 
@@ -235,14 +241,23 @@ static gn_error phonet_send_message(unsigned int messagesize, unsigned char mess
 
 	/* Now construct the message header. */
 
-	if (state->config.connection_type == GN_CT_Bluetooth) {
+	switch (state->config.connection_type) {
+	case GN_CT_Bluetooth:
 		out_buffer[current++] = FBUS_PHONET_BLUETOOTH_FRAME_ID;
 		out_buffer[current++] = FBUS_DEVICE_PHONE;
 		out_buffer[current++] = FBUS_PHONET_BLUETOOTH_DEVICE_PC;
-	} else {
+		break;
+	case GN_CT_DKU2:
+	case GN_CT_DKU2LIBUSB:
+		out_buffer[current++] = FBUS_PHONET_DKU2_FRAME_ID;
+		out_buffer[current++] = FBUS_DEVICE_PHONE;
+		out_buffer[current++] = FBUS_PHONET_DKU2_DEVICE_PC;
+		break;
+	default:
 		out_buffer[current++] = FBUS_PHONET_FRAME_ID;
 		out_buffer[current++] = FBUS_DEVICE_PHONE; /* Destination */
 		out_buffer[current++] = FBUS_DEVICE_PC;    /* Source */
+		break;
 	}
 
 	out_buffer[current++] = messagetype; /* Type */
@@ -292,6 +307,8 @@ gn_error phonet_initialise(struct gn_statemachine *state)
 	switch (state->config.connection_type) {
 	case GN_CT_Infrared:
 	case GN_CT_Irda:
+	case GN_CT_DKU2:
+	case GN_CT_DKU2LIBUSB:
 	case GN_CT_Bluetooth:
 		if (phonet_open(state) == true)
 			error = GN_ERR_NONE;
