@@ -1,10 +1,10 @@
 /*
 
-  $Id: atgen.c,v 1.107 2004/07/01 22:17:03 pkot Exp $
+  $Id: atgen.c,v 1.143 2007/05/01 23:50:14 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,14 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001 Manfred Jonsson <manfred.jonsson@gmx.de>
-  Copyright (C) 2002 Pawel Kot <pkot@linuxnews.pl>
+  Copyright (C) 2001      Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2002      Pavel Machek, Petr Cech
+  Copyright (C) 2002-2004 Pawel Kot
+  Copyright (C) 2002-2003 Ladis Michl
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003-2004 Igor Popik
+  Copyright (C) 2004      Hugo Hass, Ron Yorston
+  Copyright (C) 2007      Jeremy Laine
 
   This file provides functions specific to generic AT command compatible
   phones. See README for more details on supported mobile phones.
@@ -43,6 +49,7 @@
 #include "phones/atnok.h"
 #include "phones/atsie.h"
 #include "phones/atsoer.h"
+#include "phones/atsam.h"
 #include "links/atbus.h"
 
 static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state);
@@ -63,8 +70,12 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 /* static gn_error ReplyDeleteSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state); */
 static gn_error ReplyGetCharset(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyGetSMSCenter(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+#ifdef SECURITY
 static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+#endif
 static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 
 static gn_error AT_Identify(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetModel(gn_data *data, struct gn_statemachine *state);
@@ -89,12 +100,18 @@ static gn_error AT_DeleteSMS(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetCharset(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetCharset(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSMSCenter(gn_data *data, struct gn_statemachine *state);
+#ifdef SECURITY
 static gn_error AT_EnterSecurityCode(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSecurityCodeStatus(gn_data *data, struct gn_statemachine *state);
+#endif
 static gn_error AT_DialVoice(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_AnswerCall(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_CancelCall(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_SetCallNotification(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_SendDTMF(gn_data *data, struct gn_statemachine *state);
 
 typedef struct {
 	int gop;
@@ -130,12 +147,19 @@ static at_function_init_type at_function_init[] = {
 	{ GN_OP_AT_GetCharset,         AT_GetCharset,            ReplyGetCharset },
 	{ GN_OP_AT_SetCharset,         AT_SetCharset,            Reply },
 	{ GN_OP_GetSMSCenter,          AT_GetSMSCenter,          ReplyGetSMSCenter },
+#ifdef SECURITY
 	{ GN_OP_GetSecurityCodeStatus, AT_GetSecurityCodeStatus, ReplyGetSecurityCodeStatus },
 	{ GN_OP_EnterSecurityCode,     AT_EnterSecurityCode,     Reply },
+#endif
 	{ GN_OP_MakeCall,              AT_DialVoice,             Reply },
 	{ GN_OP_AnswerCall,            AT_AnswerCall,            Reply },
 	{ GN_OP_CancelCall,            AT_CancelCall,            Reply },
+	{ GN_OP_AT_Ring,               NULL,                     ReplyRing },
+	{ GN_OP_SetCallNotification,   AT_SetCallNotification,   Reply },
 	{ GN_OP_GetNetworkInfo,        AT_GetNetworkInfo,        ReplyGetNetworkInfo },
+	{ GN_OP_SetDateTime,           AT_SetDateTime,           Reply },
+	{ GN_OP_GetDateTime,           AT_GetDateTime,           ReplyGetDateTime },
+	{ GN_OP_SendDTMF,              AT_SendDTMF,              Reply },
 };
 
 char *strip_quotes(char *s)
@@ -164,18 +188,20 @@ static char *strip_brackets(char *s)
 	return s;
 }
 
-static void reply_simpletext(char *l1, char *l2, char *c, char *t)
+static void reply_simpletext(char *l1, char *l2, char *c, char *t, size_t maxlength)
 {
-	int i;
+	int i, n;
 
-	if ((strncmp(l1, c, 5) == 0) && (t != NULL)) {
-		if (strncmp(l2, c, 7) == 0)  {
-			for (i = 7; isspace(l2[i]); i++) ;
-			strcpy(t, strip_quotes(l2 + i));
+	n = strlen(c);
+	if ((strncmp(l1, c, n - 2) == 0) && (t != NULL)) {
+		if (strncmp(l2, c, n) == 0)  {
+			for (i = n; isspace(l2[i]); i++) ;
+			strncpy(t, strip_quotes(l2 + i), maxlength);
 		} else {
 			for (i = 0; isspace(l2[i]); i++) ;
-			strcpy(t, l2 + i);
+			strncpy(t, l2 + i, maxlength);
 		}
+		t[maxlength - 1] = '\0';
 	}
 }
 
@@ -216,6 +242,8 @@ char *memorynames[] = {
 	"CB", /* Currently selected memory */
 };
 
+#define NR_MEMORIES (sizeof (memorynames) / sizeof ((memorynames)[0]))
+
 typedef struct {
 	char *str;
 	at_charset charset;
@@ -244,47 +272,48 @@ static gn_error Functions(gn_operation op, gn_data *data, struct gn_statemachine
 }
 
 /* Functions to encode and decode strings */
-int at_encode(int charset, char *dst, char *src, int len)
+size_t at_encode(at_charset charset, char *dst, size_t dst_len, const char *src, size_t len)
 {
+	size_t ret;
+
 	switch (charset) {
 	case AT_CHAR_GSM:
-		len = char_ascii_encode(dst, src, len);
+		ret = char_ascii_encode(dst, dst_len, src, len);
 		break;
 	case AT_CHAR_HEXGSM:
-		char_hex_encode(dst, src, len);
-		len *= 2;
+		ret = char_hex_encode(dst, dst_len, src, len);
 		break;
 	case AT_CHAR_UCS2:
-		char_ucs2_encode(dst, src, len);
-		len *= 4;
+		ret = char_ucs2_encode(dst, dst_len, src, len);
 		break; 
 	default:
-		memcpy(dst, src, len);
+		memcpy(dst, src, dst_len >= len ? len : dst_len);
+		ret = len;
 		break;
 	}
-	dst[len] = '\0';
-	return len;
+	if (ret < dst_len)
+		dst[ret] = '\0';
+	return ret+1;
 }
 
 void at_decode(int charset, char *dst, char *src, int len)
 {
 	switch (charset) {
+	/* char_*_decode() functions null terminate the strings */
 	case AT_CHAR_GSM:
 		char_ascii_decode(dst, src, len);
 		break;
 	case AT_CHAR_HEXGSM:
 		char_hex_decode(dst, src, len);
-		len *= 2;
 		break;
 	case AT_CHAR_UCS2:
 		char_ucs2_decode(dst, src, len);
-		len *= 4;
 		break;
 	default:
 		memcpy(dst, src, len);
+		dst[len] = 0;
 		break;
 	}
-	dst[len] = '\0';
 }
 
 at_recv_function_type at_insert_recv_function(int type, at_recv_function_type func, struct gn_statemachine *state)
@@ -504,6 +533,8 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
 	gn_error ret = GN_ERR_NONE;
 
 	if (mt != drvinst->memorytype) {
+		if (mt >= NR_MEMORIES)
+			return GN_ERR_INVALIDMEMORYTYPE;
 		sprintf(req, "AT+CPBS=\"%s\"\r", memorynames[mt]);
 		ret = sm_message_send(13, GN_OP_Init, req, state);
 		if (ret)
@@ -519,6 +550,16 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
 	return ret;
 }
 
+/* GSM 07.05 section 3.2.3 specified the +CPMS (Preferred Message Store) AT
+ * command as taking 3 memory parameters <mem1>, <mem2> and <mem3>.  Each of
+ * these stores is used for different purposes:
+ *   <mem1> - memory from which messages are read and deleted
+ *   <mem2> - memory to which writing and sending operations are made
+ *   <mem3> - memory to which received SMs are preferred to be stored
+ *
+ * According to ETSI TS 127 005 V6.0.1 (2005-01) section 3.2.2, second and
+ * third parameter are optional, so we set only <mem1> to be more compatible.
+ */
 gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -527,12 +568,13 @@ gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 	gn_error ret = GN_ERR_NONE;
 
 	if (mt != drvinst->smsmemorytype) {
+		if (mt >= NR_MEMORIES)
+			return GN_ERR_INVALIDMEMORYTYPE;
+		gn_data_clear(&data);
 		sprintf(req, "AT+CPMS=\"%s\"\r", memorynames[mt]);
 		ret = sm_message_send(13, GN_OP_Init, req, state);
-		if (ret != GN_ERR_NONE)
-			return GN_ERR_NOTREADY;
-		gn_data_clear(&data);
-		ret = sm_block_no_retry(GN_OP_Init, &data, state);
+		if (ret == GN_ERR_NONE)
+			ret = sm_block_no_retry(GN_OP_Init, &data, state);
 		if (ret == GN_ERR_NONE)
 			drvinst->smsmemorytype = mt;
 	}
@@ -642,30 +684,70 @@ static gn_error AT_Identify(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_GetModel(gn_data *data, struct gn_statemachine *state)
 {
+	gn_error err;
+
 	if (sm_message_send(8, GN_OP_Identify, "AT+CGMM\r", state))
 		return GN_ERR_NOTREADY;
-	return sm_block_no_retry(GN_OP_Identify, data, state);
+	if ((err=sm_block_no_retry(GN_OP_Identify, data, state)) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	if (sm_message_send(7, GN_OP_Identify, "AT+GMM\r", state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_Identify, data, state) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	return err;
 }
 
 static gn_error AT_GetManufacturer(gn_data *data, struct gn_statemachine *state)
 {
+	gn_error err;
+
 	if (sm_message_send(8, GN_OP_Identify, "AT+CGMI\r", state))
 		return GN_ERR_NOTREADY;
-	return sm_block_no_retry(GN_OP_Identify, data, state);
+	if ((err = sm_block_no_retry(GN_OP_Identify, data, state)) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	if (sm_message_send(7, GN_OP_Identify, "AT+GMI\r", state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_Identify, data, state) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	return err;
 }
 
 static gn_error AT_GetRevision(gn_data *data, struct gn_statemachine *state)
 {
+	gn_error err;
+
 	if (sm_message_send(8, GN_OP_Identify, "AT+CGMR\r", state))
 		return GN_ERR_NOTREADY;
-	return sm_block_no_retry(GN_OP_Identify, data, state);
+	if ((err = sm_block_no_retry(GN_OP_Identify, data, state)) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	if (sm_message_send(7, GN_OP_Identify, "AT+GMR\r", state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_Identify, data, state) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	return err;
 }
 
 static gn_error AT_GetIMEI(gn_data *data, struct gn_statemachine *state)
 {
+	gn_error err;
+
 	if (sm_message_send(8, GN_OP_Identify, "AT+CGSN\r", state))
 		return GN_ERR_NOTREADY;
-	return sm_block_no_retry(GN_OP_Identify, data, state);
+	if ((err = sm_block_no_retry(GN_OP_Identify, data, state)) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	if (sm_message_send(7, GN_OP_Identify, "AT+GSN\r", state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_Identify, data, state) == GN_ERR_NONE)
+		return GN_ERR_NONE;
+
+	return err;
 }
 
 /* gets battery level and power source */
@@ -739,16 +821,18 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 		ret = state->driver.functions(GN_OP_AT_SetCharset, data, state);
 		if (ret)
 			return ret;
-		ofs = sprintf(req, "AT+CPBW=%d,\"%s\",%s,\"",
-			      data->phonebook_entry->location+drvinst->memoryoffset,
-			      data->phonebook_entry->number,
-			      data->phonebook_entry->number[0] == '+' ? "145" : "129");
-		len = strlen(data->phonebook_entry->name);
+		ofs = snprintf(req, sizeof(req), "AT+CPBW=%d,\"%s\",%s,\"",
+			       data->phonebook_entry->location+drvinst->memoryoffset,
+			       data->phonebook_entry->number,
+			       data->phonebook_entry->number[0] == '+' ? "145" : "129");
 		tmp = req + ofs;
-		len = at_encode(drvinst->charset, tmp, data->phonebook_entry->name, len);
-		tmp[len++] = '"'; tmp[len++] = '\r';
+		len = at_encode(drvinst->charset, tmp, sizeof(req - ofs),
+				data->phonebook_entry->name,
+				strlen(data->phonebook_entry->name));
+		tmp[len-1] = '"';
+		tmp[len++] = '\r';
 		len += ofs;
-	} 
+	}
 	if (sm_message_send(len, GN_OP_WritePhonebook, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_WritePhonebook, data, state);
@@ -777,7 +861,7 @@ static gn_error AT_DeletePhonebook(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_CallDivert(gn_data *data, struct gn_statemachine *state)
 {
-	char req[64];
+	char req[64], req2[64];
 
 	if (!data->call_divert) return GN_ERR_UNKNOWN;
 
@@ -800,14 +884,17 @@ static gn_error AT_CallDivert(gn_data *data, struct gn_statemachine *state)
 		dprintf("3. %d\n", data->call_divert->type);
 		return GN_ERR_NOTIMPLEMENTED;
 	}
-	if (data->call_divert->operation == GN_CDV_Register)
-		sprintf(req, "%s,%d,\"%s\",%d,,,%d", req,
+	if (data->call_divert->operation == GN_CDV_Register) {
+		sprintf(req2, ",%d,\"%s\",%d,,,%d",
 			data->call_divert->operation,
 			data->call_divert->number.number,
 			data->call_divert->number.type,
 			data->call_divert->timeout);
-	else
-		sprintf(req, "%s,%d", req, data->call_divert->operation);
+		strcat(req, req2);
+	} else {
+		sprintf(req2, ",%d", data->call_divert->operation);
+		strcat(req, req2);
+	}
 
 	strcat(req, "\r");
 
@@ -830,7 +917,13 @@ static gn_error AT_GetSMSStatus(gn_data *data, struct gn_statemachine *state)
 
 	if (!data->sms_status) return GN_ERR_INTERNALERROR;
 
-	ret = sm_message_send(13, GN_OP_GetSMSStatus, "AT+CPMS=\"SM\"\r", state);
+        if (data->memory_status) {
+                ret = AT_SetSMSMemoryType(data->memory_status->memory_type,  state);
+                if (ret != GN_ERR_NONE)
+                        return ret;
+        }
+
+	ret = sm_message_send(9, GN_OP_GetSMSStatus, "AT+CPMS?\r", state);
 	if (ret != GN_ERR_NONE)
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_GetSMSStatus, data, state);
@@ -855,6 +948,7 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 	unsigned char req[10240], req2[5120];
 	gn_error error;
 	unsigned int length, tmp, offset = 0;
+	at_driver_instance *drvinst = AT_DRVINST(state);
 
 	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
 
@@ -867,10 +961,14 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 	dprintf("PDU mode set\n");
 
 	/* Prepare the message and count the size */
-	memcpy(req2, data->raw_sms->message_center,
-	       data->raw_sms->message_center[0] + 1);
-	offset += data->raw_sms->message_center[0];
-
+	if(drvinst->no_smsc) {
+		/* not even a length byte included */
+		offset--;
+	} else {
+		memcpy(req2, data->raw_sms->message_center,
+		       data->raw_sms->message_center[0] + 1);
+		offset += data->raw_sms->message_center[0];
+	}
 	/* Validity period in relative format */
 	req2[offset + 1] = 0x01 | 0x10;
 	if (data->raw_sms->reject_duplicates) req2[offset + 1] |= 0x04;
@@ -896,7 +994,11 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 
 	/* Length in AT mode is the length of the full message minus
 	 * SMSC field length */
-	sprintf(req, "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
+	if(drvinst->no_smsc) {
+		sprintf(req, "AT+%s=%d\r", cmd, length);
+	} else {
+		sprintf(req, "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
+	}
 	dprintf("Sending initial sequence\n");
 	if (sm_message_send(strlen(req), GN_OP_AT_Prompt, req, state))
 		return GN_ERR_NOTREADY;
@@ -967,6 +1069,7 @@ static gn_error AT_GetSMSCenter(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetSMSCenter, data, state);
 }
 
+#ifdef SECURITY
 static gn_error AT_GetSecurityCodeStatus(gn_data *data, struct gn_statemachine *state)
 {
  	if (sm_message_send(9, GN_OP_GetSecurityCodeStatus, "AT+CPIN?\r", state))
@@ -986,6 +1089,7 @@ static gn_error AT_EnterSecurityCode(gn_data *data, struct gn_statemachine *stat
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_EnterSecurityCode, data, state);
 }
+#endif
 
 static gn_error AT_DialVoice(gn_data *data, struct gn_statemachine *state)
 {
@@ -1013,6 +1117,26 @@ static gn_error AT_CancelCall(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_CancelCall, data, state);
 }
 
+static gn_error AT_SetCallNotification(gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	gn_error err;
+
+	if (!drvinst->call_notification && !data->call_notification)
+		return GN_ERR_NONE;
+
+	if (!drvinst->call_notification) {
+		if (sm_message_send(9, GN_OP_SetCallNotification, "AT+CRC=1\r", state))
+			return GN_ERR_NOTREADY;
+		if ((err = sm_block_no_retry(GN_OP_SetCallNotification, data, state)) != GN_ERR_NONE)
+			return err;
+	}
+
+	drvinst->call_notification = data->call_notification;
+
+	return GN_ERR_NONE;
+}
+
 static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error;
@@ -1037,6 +1161,67 @@ static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetNetworkInfo, data, state);
 }
 
+static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state)
+{
+	char req[29];
+
+	gn_timestamp *dt = data->datetime;
+
+	sprintf(req, "AT+CCLK=\"%02d/%02d/%02d,%02d:%02d:%02d\"\r",
+		dt->year % 100, dt->month, dt->day,
+		dt->hour, dt->minute, dt->second);
+
+	if (sm_message_send(28, GN_OP_SetDateTime, req, state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_SetDateTime, data, state);
+}
+
+static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(9, GN_OP_GetDateTime, "AT+CCLK?\r", state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_GetDateTime, data, state);
+}
+
+
+/*
+ * This command allows to send DTMF tones and arbitrary tones.
+ * DTMF is a single ASCII character in the set 0-9, *, #, A-D
+ * FIXME: tone and duration parameters are not yet supported
+ */
+static gn_error AT_SendDTMF(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error = GN_ERR_NONE;
+	unsigned char req[32];
+	int len, i, dtmf_len;
+
+	if (!data || !data->dtmf_string)
+		return GN_ERR_INTERNALERROR;
+
+	dtmf_len = strlen(data->dtmf_string);
+	if (dtmf_len < 1)
+		return GN_ERR_WRONGDATAFORMAT;
+
+	/* First let's check out if the command is supported by the phone */
+	len = snprintf(req, sizeof(req), "AT+VTS=?\r");
+	if (sm_message_send(len, GN_OP_SendDTMF, req, state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_SendDTMF, data, state) != GN_ERR_NONE)	
+		return GN_ERR_NOTSUPPORTED;
+
+	/* Send it char by char */
+	for (i = 0; i < dtmf_len; i++) {
+		len = snprintf(req, sizeof(req), "AT+VTS=%c\r", data->dtmf_string[i]);
+		if (sm_message_send(len, GN_OP_SendDTMF, req, state))
+			return GN_ERR_NOTREADY;
+		error = sm_block_no_retry(GN_OP_SendDTMF, data, state);
+		if (error)
+			break;
+	}
+
+	return error;
+}
+
 static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1060,18 +1245,20 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 		if (data->phonebook_entry) {
 			*(data->phonebook_entry->number) = '\0';
 			*(data->phonebook_entry->name) = '\0';
-			data->phonebook_entry->caller_group = 0;
+			data->phonebook_entry->caller_group = GN_PHONEBOOK_GROUP_None;
 			data->phonebook_entry->subentries_count = 0;
 			data->phonebook_entry->empty = true;
 		}
 		return GN_ERR_NONE;
 	}
 	if (data->phonebook_entry) {
-		data->phonebook_entry->caller_group = 0;
+		data->phonebook_entry->caller_group = GN_PHONEBOOK_GROUP_None;
 		data->phonebook_entry->subentries_count = 0;
+		data->phonebook_entry->empty = false;
 
 		/* store number */
 		pos = strchr(buf.line2, '\"');
+		dprintf("NUMBER: %s\n", pos);
 		endpos = NULL;
 		if (pos)
 			endpos = strchr(++pos, '\"');
@@ -1084,6 +1271,7 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 		pos = NULL;
 		if (endpos)
 			pos = strchr(endpos+2, ',');
+		dprintf("NAME: %s\n", pos);
 		endpos = NULL;
 		if (pos) {
 			pos = strip_quotes(pos+1);
@@ -1270,16 +1458,26 @@ static gn_error ReplyIdentify(int messagetype, unsigned char *buffer, int length
 	at_line_buffer buf;
 	gn_error error;
 
-	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
 
 	buf.line1 = buffer + 1;
 	buf.length = length;
 	splitlines(&buf);
 	if (!strncmp(buf.line1, "AT+CG", 5)) {
-		reply_simpletext(buf.line1+2, buf.line2, "+CGSN: ", data->imei);
-		reply_simpletext(buf.line1+2, buf.line2, "+CGMM: ", data->model);
-		reply_simpletext(buf.line1+2, buf.line2, "+CGMI: ", data->manufacturer);
-		reply_simpletext(buf.line1+2, buf.line2, "+CGMR: ", data->revision);
+		reply_simpletext(buf.line1+2, buf.line2, "+CGSN: ", data->imei, GN_IMEI_MAX_LENGTH);
+		if (!data->model[0])
+			reply_simpletext(buf.line1+2, buf.line2, "+CGMM: ", data->model, GN_MODEL_MAX_LENGTH);
+		reply_simpletext(buf.line1+2, buf.line2, "+CGMI: ", data->manufacturer, GN_MANUFACTURER_MAX_LENGTH);
+		reply_simpletext(buf.line1+2, buf.line2, "+CGMR: ", data->revision, GN_REVISION_MAX_LENGTH);
+		if (!data->model[0])
+			reply_simpletext(buf.line1+2, buf.line4, "+CGMR: ", data->model, GN_MODEL_MAX_LENGTH);
+	} else if (!strncmp(buf.line1, "AT+G", 4)) {
+		reply_simpletext(buf.line1+2, buf.line2, "+GSN: ", data->imei, GN_IMEI_MAX_LENGTH);
+		if (!data->model[0])
+			reply_simpletext(buf.line1+2, buf.line2, "+GMM: ", data->model, GN_MODEL_MAX_LENGTH);
+		reply_simpletext(buf.line1+2, buf.line2, "+GMI: ", data->manufacturer, GN_MANUFACTURER_MAX_LENGTH);
+		reply_simpletext(buf.line1+2, buf.line2, "+GMR: ", data->revision, GN_REVISION_MAX_LENGTH);
 	}
 	return GN_ERR_NONE;
 }
@@ -1314,7 +1512,7 @@ static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int le
 	buf.length = length;
 	splitlines(&buf);
 
-	if (sscanf(buf.line2, "+CPMS: %d", &data->sms_status->number) != 1)
+	if (sscanf(buf.line2, "+CPMS: \"%*c%*c\",%d,%*d", &data->sms_status->number) != 1)
 		return GN_ERR_FAILED;
 
 	data->sms_status->unread = 0;
@@ -1350,19 +1548,30 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	at_line_buffer buf;
 	gn_error ret = GN_ERR_NONE;
 	unsigned int sms_len, l, offset = 0;
-	char *tmp;
+	unsigned char *tmp;
 	gn_error error;
+ 	at_driver_instance *drvinst = AT_DRVINST(state);
 
-	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
 	
 	buf.line1 = buffer + 1;
 	buf.length = length;
 
 	splitlines(&buf);
 
-	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+	if (!data->raw_sms)
+		return GN_ERR_INTERNALERROR;
 
 	tmp = strrchr(buf.line2, ',');
+	/* The following sequence is correct for emtpy location:
+	 * w: AT+CMGR=9
+	 * r: AT+CMGR=9
+	 *  :
+	 *  : OK
+	 */
+	if (!tmp)
+		return GN_ERR_EMPTYLOCATION;
 	sms_len = atoi(tmp+1);
 	if (sms_len == 0)
 		return GN_ERR_EMPTYLOCATION;
@@ -1375,18 +1584,23 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	}
 	dprintf("%s\n", buf.line3);
 	hex2bin(tmp, buf.line3, sms_len);
-	l = tmp[offset] + 1;
-	if (l > sms_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
-		dprintf("Invalid message center length (%d)\n", l);
-		ret = GN_ERR_INTERNALERROR;
-		goto out;
+
+	if(!drvinst->no_smsc) {
+		l = tmp[offset] + 1;
+		if (l > sms_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
+			dprintf("Invalid message center length (%d)\n", l);
+			ret = GN_ERR_INTERNALERROR;
+			goto out;
+		}
+		memcpy(data->raw_sms->message_center, tmp, l);
+		offset += l;
 	}
-	memcpy(data->raw_sms->message_center, tmp, l);
-	offset += l;
+
 	data->raw_sms->type                = (tmp[offset] & 0x03) << 1;
-	data->raw_sms->udh_indicator       = tmp[offset];
 	data->raw_sms->more_messages       = tmp[offset];
-	data->raw_sms->report_status       = tmp[offset];
+	data->raw_sms->reply_via_same_smsc = tmp[offset];
+	data->raw_sms->udh_indicator       = tmp[offset];
+	data->raw_sms->report              = tmp[offset];
 	l = (tmp[offset + 1] % 2) ? tmp[offset + 1] + 1 : tmp[offset + 1] ;
 	l = l / 2 + 2;
 	if (l + offset + 11 > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
@@ -1396,14 +1610,16 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	}
 	memcpy(data->raw_sms->remote_number, tmp + offset + 1, l);
 	offset += l;
-	data->raw_sms->reply_via_same_smsc = 0;
 	data->raw_sms->reject_duplicates   = 0;
-	data->raw_sms->report              = 0;
+	data->raw_sms->report_status       = 0;
 	data->raw_sms->reference           = 0;
 	data->raw_sms->pid                 = tmp[offset + 1];
 	data->raw_sms->dcs                 = tmp[offset + 2];
 	memcpy(data->raw_sms->smsc_time, tmp + offset + 3, 7);
-	data->raw_sms->length              = tmp[offset + 10] & 0x00ff;
+	data->raw_sms->length              = tmp[offset + 10];
+	data->raw_sms->user_data_length = data->raw_sms->length;
+	if (data->raw_sms->udh_indicator & 0x40)
+		data->raw_sms->user_data_length -= tmp[offset+11] + 1;
 	if (sms_len - offset - 11 > 1000) {
 		dprintf("Phone gave as poisonous (too short?) reply %s, either phone went crazy or communication went out of sync\n", buf.line3);
 		ret = GN_ERR_INTERNALERROR;
@@ -1456,6 +1672,7 @@ static gn_error ReplyGetCharset(int messagetype, unsigned char *buffer, int leng
 	return GN_ERR_FAILED;
 }
 
+#ifdef SECURITY
 static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
@@ -1499,6 +1716,56 @@ static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffe
 	}
 	return GN_ERR_NONE;
 }
+#endif
+
+static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	at_line_buffer buf;
+	char *pos;
+	gn_call_info cinfo;
+
+	if (!drvinst->call_notification) return GN_ERR_UNSOLICITED;
+
+	buf.line1 = buffer;
+	buf.length= length;
+	splitlines(&buf);
+
+	memset(&cinfo, 0, sizeof(cinfo));
+	cinfo.call_id = 1;
+
+	if (!strncmp(buf.line1, "RING", 4))
+		return GN_ERR_INTERNALERROR; /* AT+CRC=1 disables RING */
+
+	else if (!strncmp(buf.line1, "+CRING: ", 8)) {
+		pos = buf.line1 + 8;
+		if (!strncmp(pos, "VOICE", 5))
+			cinfo.type = GN_CALL_Voice;
+		else
+			return GN_ERR_UNHANDLEDFRAME;
+
+		drvinst->call_notification(GN_CALL_Incoming, &cinfo, state);
+
+	} else if (!strncmp(buf.line1, "CONNECT", 7))
+		drvinst->call_notification(GN_CALL_Established, &cinfo, state);
+
+	else if (!strncmp(buf.line1, "BUSY", 4))
+		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+
+	else if (!strncmp(buf.line1, "NO ANSWER", 9))
+		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+
+	else if (!strncmp(buf.line1, "NO CARRIER", 10))
+		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+
+	else if (!strncmp(buf.line1, "NO DIALTONE", 11))
+		drvinst->call_notification(GN_CALL_LocalHangup, &cinfo, state);
+
+	else
+		return GN_ERR_UNHANDLEDFRAME;
+
+	return GN_ERR_UNSOLICITED;
+}
 
 static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
@@ -1568,9 +1835,7 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 		format = atoi(strings[1]);
 		switch (format) {
 		case 0: /* network operator name given */
-			pos = strings[2];
-			pos++;
-			pos = strtok(pos, "\"");
+			pos = strip_quotes(strings[2]);
 			at_decode(drvinst->charset, tmp, pos, strlen(pos));
 			snprintf(data->network_info->network_code, sizeof(data->network_info->network_code), gn_network_code_get(tmp));
 			break;
@@ -1604,6 +1869,32 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 	return GN_ERR_NONE;
 }
 
+static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_line_buffer buf;
+	gn_error error;
+	gn_timestamp *dt;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
+
+	buf.line1 = buffer + 1;
+	buf.length= length;
+
+	splitlines(&buf);
+
+	dt = data->datetime;
+	if (sscanf(buf.line2, "+CCLK: \"%d/%d/%d,%d:%d:%d\"",
+		   &dt->year, &dt->month, &dt->day,
+		   &dt->hour, &dt->minute, &dt->second) != 6)
+		return GN_ERR_FAILED;
+
+	if (dt->year < 100)
+		dt->year += 2000;
+
+	return GN_ERR_NONE;
+}
+
 /* General reply function for phone responses. buffer[0] holds the compiled
  * success of the result (OK, ERROR, ... ). see links/atbus.h and links/atbus.c 
  * for reference */
@@ -1617,12 +1908,16 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	at_driver_instance *drvinst;
 	gn_data data;
 	gn_error ret = GN_ERR_NONE;
-	char model[20];
-	char manufacturer[20];
+	char model[GN_MODEL_MAX_LENGTH];
+	char manufacturer[GN_MANUFACTURER_MAX_LENGTH];
 	int i;
 
 	dprintf("Initializing AT capable mobile phone ...\n");
-	
+
+	/* initialize variables */
+	memset(model, 0, GN_MODEL_MAX_LENGTH);
+	memset(manufacturer, 0, GN_MANUFACTURER_MAX_LENGTH);
+
 	/* Copy in the phone info */
 	memcpy(&(state->driver), &driver_at, sizeof(gn_driver));
 
@@ -1638,6 +1933,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	drvinst->smsmemorytype = GN_MT_XX;
 	drvinst->defaultcharset = AT_CHAR_UNKNOWN;
 	drvinst->charset = AT_CHAR_UNKNOWN;
+	drvinst->no_smsc = 0;
+	drvinst->call_notification = NULL;
 
 	drvinst->if_pos = 0;
 	for (i = 0; i < GN_OP_AT_Max; i++) {
@@ -1696,6 +1993,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 		at_siemens_init(model, setupdata->model, state);
 	else if (!strncasecmp(manufacturer, "sony ericsson", 14))
 		at_sonyericsson_init(model, setupdata->model, state);
+	else if (!strncasecmp(manufacturer, "samsung", 7))
+		at_samsung_init(model, setupdata->model, state);
 	
 	StoreDefaultCharset(state);
 
@@ -1703,8 +2002,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 out:
 	if (ret) {
 		dprintf("Initialization failed (%d)\n", ret);
-		free(AT_DRVINST(state));
-		AT_DRVINST(state) = NULL;
+		/* ignore return value from GN_OP_Terminate, will use previous error code instead */
+		state->driver.functions(GN_OP_Terminate, &data, state);
 	}
 	return ret;
 }
@@ -1721,28 +2020,36 @@ static gn_error Terminate(gn_data *data, struct gn_statemachine *state)
 void splitlines(at_line_buffer *buf)
 {
 	char *pos;
+	int length = buf->length;
 
-	pos = findcrlf(buf->line1, 0, buf->length);
+	pos = findcrlf(buf->line1, 0, length);
 	if (pos) {
 		*pos = 0;
 		buf->line2 = skipcrlf(++pos);
+		length -= (buf->line2 - buf->line1);
 	} else {
 		buf->line2 = buf->line1;
 	}
-	pos = findcrlf(buf->line2, 1, buf->length);
+	pos = findcrlf(buf->line2, 1, length);
 	if (pos) {
 		*pos = 0;
 		buf->line3 = skipcrlf(++pos);
+		length -= (buf->line3 - buf->line2);
 	} else {
 		buf->line3 = buf->line2;
 	}
-	pos = findcrlf(buf->line3, 1, buf->length);
+	pos = findcrlf(buf->line3, 1, length);
 	if (pos) {
 		*pos = 0;
 		buf->line4 = skipcrlf(++pos);
+		length -= (buf->line4 - buf->line3);
 	} else {
 		buf->line4 = buf->line3;
 	}
+	pos = findcrlf(buf->line4, 1, length);
+	if (pos) {
+		*pos = 0;
+	}
 }
 
 /*
@@ -1769,7 +2076,7 @@ char *findcrlf(unsigned char *str, int test, int max)
 {
 	if (str == NULL)
 		return str;
-	while ((*str != '\n') && (*str != '\r') && ((*str != '\0') || test) && (max > 0)) {
+	while ((max > 0) && (*str != '\n') && (*str != '\r') && ((*str != '\0') || test)) {
 		str++;
 		max--;
 	}
