@@ -1,10 +1,10 @@
 /*
 
-  $Id: compat.h,v 1.36 2004/02/23 23:18:30 uid66843 Exp $
+  $Id: compat.h,v 1.54 2007/05/08 19:41:35 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,10 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002, BORBELY Zoltan
+  Copyright (C) 1999-2000 Hugh Blemings, Pavel Janik
+  Copyright (C) 2002-2004 BORBELY Zoltan, Pawel Kot
+  Copyright (C) 2002      Feico de Boer, Markus Plail
+  Copyright (C) 2003      Marcus Godehardt, Ladis Michl
 
   Header file for various platform compatibility.
 
@@ -37,8 +40,13 @@
 
 #ifdef WIN32
 #  include <windows.h>
-#  include <string.h>
-#  include <direct.h>
+#  include <locale.h>
+#  ifdef HAVE_DIRECT_H
+#    include <direct.h>
+#  endif
+#  ifdef HAVE_SYS_STAT_H
+#    include <sys/stat.h>
+#  endif
 #endif
 
 #ifdef HAVE_STDARG_H
@@ -52,6 +60,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
+#  ifndef _GNU_SOURCE
+#    define _GNU_SOURCE 1
+#  endif
 #  include <string.h>
 #endif
 
@@ -87,6 +98,22 @@
 #  include <wchar.h>
 #endif
 
+#ifdef HAVE_LIMITS_H
+#  include <limits.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
+#ifdef HAVE_TERMIOS_H
+#  include <termios.h>
+#endif
+
+#if !defined(INT_MAX)
+#  define INT_MAX 2147483647
+#endif
+
 /*
  * The following ifdef block is the standard way of creating macros which make
  * exporting from a DLL simpler. All files within this DLL are compiled with 
@@ -98,16 +125,23 @@
  */
 #if defined(WIN32)
 #  if defined(GNOKIIDLL_EXPORTS) || defined(_USRDLL) || defined(DLL_EXPORT)
-#    define API __declspec(dllexport)
+#    define GNOKII_API __declspec(dllexport)
 #  elif defined(GNOKIIDLL_IMPORTS)
-#    define API __declspec(dllimport)
+#    define GNOKII_API __declspec(dllimport)
 #  else
-#    define API
+#    define GNOKII_API
 #  endif
-#else /* !WIN32 */
-#  define API
+#elif (__GNUC__ - 0 > 3 || __GNUC__ == 3 && __GNUC_MINOR__ > 3)
+#    define GNOKII_API __attribute__ ((visibility("default")))
+#else
+#    define GNOKII_API
 #endif /* WIN32 */
 
+/* I assume that HAVE_STRNDUP always implies HAVE_STRING_H */
+#ifndef HAVE_STRNDUP
+extern char *strndup(const char *src, size_t n);
+#endif
+
 #ifndef	HAVE_TIMEOPS
 
 #undef timerisset
@@ -172,6 +206,10 @@ int asprintf(char **ptr, const char *format, ...);
 int vasprintf(char **ptr, const char *format, va_list ap);
 #endif
 
+#ifndef HAVE_TIMEGM
+time_t timegm(struct tm *tm);
+#endif
+
 /*
  * The following code was taken from W. Richard Stevens'
  * "UNIX Network Programming", Volume 1, Second Edition.
@@ -209,7 +247,7 @@ int vasprintf(char **ptr, const char *format, va_list ap);
 #endif
 
 /* Get rid of long defines. Use #if __unices__ */
-#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__)
+#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__) || defined(__NetBSD__)
 
 /* This one is for NLS. */
 #ifdef ENABLE_NLS
@@ -238,6 +276,11 @@ int vasprintf(char **ptr, const char *format, va_list ap);
 /* for Linux Bluetooth compability */
 #if !defined(HAVE_STDINT_H) && !defined(HAVE_INTTYPES_H)
 	typedef unsigned char uint8_t;
+	typedef unsigned short uint16_t;
+	typedef unsigned int uint32_t;
+	typedef char int8_t;
+	typedef short int16_t;
+	typedef int int32_t;
 #endif
 
 #ifdef HAVE_WCRTOMB
@@ -250,4 +293,14 @@ int vasprintf(char **ptr, const char *format, va_list ap);
 #  define MBSTATE_DEC_CLEAR(x) wctomb(NULL, 0)
 #endif
 
+/* MS Visual C does not have va_copy */
+#ifndef va_copy
+#  ifdef __va_copy
+#    define va_copy(dest, src)    __va_copy((dest), (src))
+#  else
+#    define va_copy(dest, src)    memcpy(&(dest), &(src), sizeof(va_list))
+#  endif
+#endif
+
+
 #endif
