@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-internal.h,v 1.34 2006/08/06 20:42:21 pkot Exp $
+  $Id: gnokii-internal.h,v 1.35 2007/04/14 19:36:35 pkot Exp $
 
   G N O K I I
 
@@ -85,18 +85,18 @@ unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src,
 unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src, int len);
 
 void char_ascii_decode(unsigned char* dest, const unsigned char* src, int len);
-unsigned int char_ascii_encode(unsigned char* dest, const unsigned char* src, unsigned int len);
+size_t char_ascii_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
 void char_hex_decode(unsigned char* dest, const unsigned char* src, int len);
-void char_hex_encode(unsigned char* dest, const unsigned char* src, int len);
+size_t char_hex_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
 void char_ucs2_decode(unsigned char* dest, const unsigned char* src, int len);
-void char_ucs2_encode(unsigned char* dest, const unsigned char* src, int len);
+size_t char_ucs2_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
 unsigned char char_def_alphabet_encode(unsigned char value);
 unsigned char char_def_alphabet_decode(unsigned char value);
 
-int char_uni_alphabet_encode(unsigned char const *value, wchar_t *dest, MBSTATE *mbs);
+size_t char_uni_alphabet_encode(const char *value, size_t n, wchar_t *dest, MBSTATE *mbs);
 int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs);
 
 extern char *char_bcd_number_get(u8 *number);
