@@ -1,10 +1,10 @@
 /*
 
-  $Id: gnapplet.h,v 1.12 2004/06/20 00:11:47 bozo Exp $
+  $Id: gnapplet.h,v 1.21 2006/04/23 10:47:32 deller Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -36,7 +36,7 @@
 
 
 #define	GNAPPLET_MAJOR_VERSION	0
-#define	GNAPPLET_MINOR_VERSION	11
+#define	GNAPPLET_MINOR_VERSION	18
 
 #define	GNAPPLET_MSG_INFO	1
 #define	GNAPPLET_MSG_PHONEBOOK	2
@@ -44,6 +44,9 @@
 #define	GNAPPLET_MSG_POWER	4
 #define	GNAPPLET_MSG_DEBUG	5
 #define	GNAPPLET_MSG_SMS	6
+#define	GNAPPLET_MSG_CALENDAR	7
+#define	GNAPPLET_MSG_CLOCK	8
+#define	GNAPPLET_MSG_PROFILE	9
 
 #define	GNAPPLET_MSG_INFO_ID_REQ	1
 #define	GNAPPLET_MSG_INFO_ID_RESP	2
@@ -85,16 +88,49 @@
 #define	GNAPPLET_MSG_SMS_MESSAGE_SEND_RESP	16
 #define	GNAPPLET_MSG_SMS_MESSAGE_DELETE_REQ	17
 #define	GNAPPLET_MSG_SMS_MESSAGE_DELETE_RESP	18
-#define	GNAPPLET_MSG_SMS_CENTER_READ_REQ	19
-#define	GNAPPLET_MSG_SMS_CENTER_READ_RESP	20
-#define	GNAPPLET_MSG_SMS_CENTER_WRITE_REQ	21
-#define	GNAPPLET_MSG_SMS_CENTER_WRITE_RESP	22
+#define	GNAPPLET_MSG_SMS_MESSAGE_MOVE_REQ	19
+#define	GNAPPLET_MSG_SMS_MESSAGE_MOVE_RESP	20
+#define	GNAPPLET_MSG_SMS_CENTER_READ_REQ	21
+#define	GNAPPLET_MSG_SMS_CENTER_READ_RESP	22
+#define	GNAPPLET_MSG_SMS_CENTER_WRITE_REQ	23
+#define	GNAPPLET_MSG_SMS_CENTER_WRITE_RESP	24
+
+#define	GNAPPLET_MSG_CALENDAR_NOTE_READ_REQ	1
+#define	GNAPPLET_MSG_CALENDAR_NOTE_READ_RESP	2
+#define	GNAPPLET_MSG_CALENDAR_NOTE_WRITE_REQ	3
+#define	GNAPPLET_MSG_CALENDAR_NOTE_WRITE_RESP	4
+#define	GNAPPLET_MSG_CALENDAR_NOTE_DELETE_REQ	5
+#define	GNAPPLET_MSG_CALENDAR_NOTE_DELETE_RESP	6
+#define	GNAPPLET_MSG_CALENDAR_TODO_READ_REQ	7
+#define	GNAPPLET_MSG_CALENDAR_TODO_READ_RESP	8
+#define	GNAPPLET_MSG_CALENDAR_TODO_WRITE_REQ	9
+#define	GNAPPLET_MSG_CALENDAR_TODO_WRITE_RESP	10
+#define	GNAPPLET_MSG_CALENDAR_TODO_DELETE_REQ	11
+#define	GNAPPLET_MSG_CALENDAR_TODO_DELETE_RESP	12
+
+#define	GNAPPLET_MSG_CLOCK_DATETIME_READ_REQ	1
+#define	GNAPPLET_MSG_CLOCK_DATETIME_READ_RESP	2
+#define	GNAPPLET_MSG_CLOCK_DATETIME_WRITE_REQ	3
+#define	GNAPPLET_MSG_CLOCK_DATETIME_WRITE_RESP	4
+#define	GNAPPLET_MSG_CLOCK_ALARM_READ_REQ	5
+#define	GNAPPLET_MSG_CLOCK_ALARM_READ_RESP	6
+#define	GNAPPLET_MSG_CLOCK_ALARM_WRITE_REQ	7
+#define	GNAPPLET_MSG_CLOCK_ALARM_WRITE_RESP	8
+
+#define	GNAPPLET_MSG_PROFILE_READ_REQ		1
+#define	GNAPPLET_MSG_PROFILE_READ_RESP		2
+#define	GNAPPLET_MSG_PROFILE_WRITE_REQ		3
+#define	GNAPPLET_MSG_PROFILE_WRITE_RESP		4
+#define	GNAPPLET_MSG_PROFILE_GET_ACTIVE_REQ	5
+#define	GNAPPLET_MSG_PROFILE_GET_ACTIVE_RESP	6
+#define	GNAPPLET_MSG_PROFILE_SET_ACTIVE_REQ	7
+#define	GNAPPLET_MSG_PROFILE_SET_ACTIVE_RESP	8
 
 
 typedef struct {
 	int proto_major;
 	int proto_minor;
-	char manufacturer[20];
+	char manufacturer[GN_MANUFACTURER_MAX_LENGTH];
 	char model[GN_MODEL_MAX_LENGTH];
 	char imei[GN_IMEI_MAX_LENGTH];
 	char sw_version[GN_REVISION_MAX_LENGTH];
