@@ -1,10 +1,10 @@
 /*
 
-  $Id: sms-nokia.h,v 1.4 2002/12/09 00:33:48 pkot Exp $
+  $Id: sms-nokia.h,v 1.5 2005/04/24 19:03:37 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002 Pawe Kot <pkot@linuxnews.pl>
+  Copyright (C) 2002 Pawel Kot
 
   Various function prototypes used by Nokia in their SMS extension protocols.
 
@@ -37,8 +37,8 @@
 #define GN_SMS_MULTIPART_UNICODE      1
 #define GN_SMS_MULTIPART_BITMAP       2
 #define GN_SMS_MULTIPART_RINGTONE     3
-#define GN_SMS_MULTIPART_PROFILENAME  4 /* This is sick. Isn't it? */
-#define GN_SMS_MULTIPART_RESERVED     5 /* I am really afraid what else can appear here... */
+#define GN_SMS_MULTIPART_PROFILENAME  4
+#define GN_SMS_MULTIPART_RESERVED     5
 #define GN_SMS_MULTIPART_SCREENSAVER  6
 
 #endif /* _gnokii_sms_nokia_h */
