@@ -1,10 +1,10 @@
 /*
 
-  $Id: smsd.c,v 1.40 2004/06/01 21:45:50 pkot Exp $
+  $Id: smsd.c,v 1.56 2006/10/01 15:25:52 ja Exp $
 
   S M S D
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix tool for the mobile phones.
 
   This file is part of gnokii.
 
@@ -23,18 +23,20 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jan Derfinak <ja@mail.upjs.sk>.
+  Copyright (C) 1999-2005 Jan Derfinak
 
 */
 
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include <stdlib.h>
 #include <pthread.h>
 #include <getopt.h>
 #include <time.h>
 #include <dlfcn.h>
 
+
 #ifndef WIN32
 # include <unistd.h>  /* for usleep */
 # include <signal.h>
@@ -51,10 +53,15 @@
 #include "misc.h"
 
 #include "gnokii.h"
+#include "config.h"
 #include "smsd.h"
 #include "lowlevel.h"
 #include "db.h"
 
+#ifdef ENABLE_NLS
+#  include <locale.h>
+#endif
+
 
 /* Hold main configuration data for smsd */
 SmsdConfig smsdConfig;
@@ -74,27 +81,6 @@ static pthread_t db_monitor_th;
 pthread_mutex_t db_monitorMutex;
 static volatile bool db_monitor;
 
-/* Escapes ' and \ with \. */
-/* Returned value needs to be free with g_free(). */
-gchar *strEscape (const gchar *const s)
-{
-  GString *str = g_string_new (s);
-  register gint i = 0;
-  gchar *ret;
-  
-  while (str->str[i] != '\0')
-  {
-    if (str->str[i] == '\\' || str->str[i] == '\'')
-      g_string_insert_c (str, i++, '\\');
-    i++;
-  }
-  
-  ret = str->str;
-  g_string_free (str, FALSE);
-  
-  return (ret);
-}
-
 
 gint LoadDB (void)
 {
@@ -118,35 +104,35 @@ gint LoadDB (void)
   DB_Bye = dlsym(handle, "DB_Bye");
   if ((error = dlerror ()) != NULL)
   {
-    g_print (error);
+    g_print ("%s\n", error);
     return (2);
   }
 
   DB_ConnectInbox = dlsym(handle, "DB_ConnectInbox");
   if ((error = dlerror ()) != NULL)
   {
-    g_print (error);
+    g_print ("%s\n", error);
     return (2);
   }
 
   DB_ConnectOutbox = dlsym(handle, "DB_ConnectOutbox");
   if ((error = dlerror ()) != NULL)
   {
-    g_print (error);
+    g_print ("%s\n", error);
     return (2);
   }
   
   DB_InsertSMS = dlsym(handle, "DB_InsertSMS");
   if ((error = dlerror ()) != NULL)
   {
-    g_print (error);
+    g_print ("%s\n", error);
     return (2);
   }
   
   DB_Look = dlsym(handle, "DB_Look");
   if ((error = dlerror ()) != NULL)
   {
-    g_print (error);
+    g_print ("%s\n", error);
     return (2);
   }
   
@@ -156,7 +142,7 @@ gint LoadDB (void)
 
 static void Version (void)
 {
-  g_print ("\nsmsd - version 1.2-cvs		(20031005)\nCopyright  Jan Derfinak  <ja@mail.upjs.sk>\n");
+  g_print ("\nsmsd - version 1.4-cvs		(20041011)\nCopyright  Jan Derfinak  <ja@mail.upjs.sk>\n");
 }
 
 
@@ -168,10 +154,13 @@ static void Usage (gchar *p)
              "            -p, --password db_password\n"
              "            -d, --db db_name\n"
              "            -c, --host db_hostname OR spool directory if -m file\n"
+             "            -s, --schema db_schema\n"
              "            -m, --module db_module (pq, mysql, file)\n"
              "            -l, --libdir path_to_db_module\n"
              "            -f, --logfile file\n"
              "            -t, --phone phone_number\n"
+             "            -i, --interval polling_interval_for_incoming_sms's_in_seconds\n"
+             "            -S, --maxsms number_of_sms's (only in dumb mode)\n"
              "            -v, --version\n"
              "            -h, --help\n"), p);
 }
@@ -191,7 +180,7 @@ static void LogFile (gchar *str, ...)
     f = stdout;
   else if ((f = fopen (smsdConfig.logFile, "a")) == NULL)
   {
-    g_print (_("Warning: Cannot open file %s for appendig.\n"), smsdConfig.logFile);
+    g_print (_("WARNING: Cannot open file %s for appending.\n"), smsdConfig.logFile);
     return;
   }
 
@@ -216,11 +205,15 @@ static void ReadConfig (gint argc, gchar *argv[])
   connection.password = g_strdup ("");
   connection.db = g_strdup ("sms");
   connection.host = g_strdup ("");
+  connection.schema = g_strdup ("public");
   smsdConfig.dbMod = g_strdup ("pq");
   smsdConfig.libDir = g_strdup (MODULES_DIR);
   smsdConfig.logFile = NULL;
   smsdConfig.phone = g_strdup ("");
+  smsdConfig.refreshInt = 1;     // Phone querying interval in seconds
+  smsdConfig.maxSMS = 10;        // Smsd uses it if GetSMSStatus isn't implemented
   smsdConfig.smsSets = 0;
+  smsdConfig.memoryType = GN_MT_XX;
 
   while (1)
   {
@@ -232,15 +225,20 @@ static void ReadConfig (gint argc, gchar *argv[])
       {"password", 1, 0, 'p'},
       {"db", 1, 0, 'd'},
       {"host", 1, 0, 'c'},
+      {"schema", 1, 0, 's'},
       {"module", 1, 0, 'm'},
       {"libdir", 1, 0, 'l'},
       {"logfile", 1, 0, 'f'},
       {"phone", 1, 0, 't'},
       {"version", 0, 0, 'v'},
-      {"help", 0, 0, 'h'}
+      {"interval", 1, 0, 'i'},
+      {"maxsms", 1, 0, 'S'},
+      {"inbox", 1, 0, 'b'},
+      {"help", 0, 0, 'h'},
+      {0, 0, 0, 0}
     };
     
-    c = getopt_long (argc, argv, "u:p:d:c:m:l:f:t:vh", longOptions, &optionIndex);
+    c = getopt_long (argc, argv, "u:p:d:c:s:m:l:f:t:vi:S:b:h", longOptions, &optionIndex);
     if (c == EOF)
       break;
     switch (c)
@@ -268,6 +266,12 @@ static void ReadConfig (gint argc, gchar *argv[])
         connection.host = g_strdup (optarg);
         memset (optarg, 'x', strlen (optarg));
         break;
+      
+      case 's':
+        g_free (connection.schema);
+        connection.schema = g_strdup (optarg);
+        memset (optarg, 'x', strlen (optarg));
+        break;
         
       case 'm':
         g_free (smsdConfig.dbMod);
@@ -291,6 +295,18 @@ static void ReadConfig (gint argc, gchar *argv[])
         smsdConfig.phone = g_strdup (optarg);
         break;
 
+      case 'i':
+        smsdConfig.refreshInt = atoi (optarg);
+        break;
+
+      case 'S':
+        smsdConfig.maxSMS = atoi (optarg);
+        break;
+
+      case 'b':
+        smsdConfig.memoryType = gn_str2memory_type (optarg);
+        break;
+
       case 'v':
         Version ();
         exit (0);
@@ -309,7 +325,7 @@ static void ReadConfig (gint argc, gchar *argv[])
   
   if ((argc - optind) != 0)
   {
-    g_print ("Wrong argument number!\n");
+    g_print (_("Wrong number of arguments\n"));
     Usage (argv[0]);
     exit (1);
   }
@@ -320,9 +336,6 @@ static void ReadConfig (gint argc, gchar *argv[])
               smsdConfig.dbMod, smsdConfig.libDir);
     exit (-2);
   }
-  
-  if (gn_cfg_read_default() < 0)
-    exit (-1);
 }
 
 
@@ -352,7 +365,26 @@ static void *SendSMS (void *a)
 }
 
 
-gint WriteSMS (gn_sms *sms)
+static void MainExit (gint sig)
+{
+  PhoneEvent *e = (PhoneEvent *) g_malloc (sizeof (PhoneEvent));
+
+  e->event = Event_Exit;
+  e->data = NULL;
+  InsertEvent (e);
+  
+  pthread_mutex_lock (&db_monitorMutex);
+  db_monitor = FALSE;
+  pthread_mutex_unlock (&db_monitorMutex);
+  
+  pthread_join (monitor_th, NULL);
+  pthread_join (db_monitor_th, NULL);
+  (*DB_Bye) ();
+  exit (sig);
+}
+
+
+GNOKII_API gint WriteSMS (gn_sms *sms)
 {
   gn_error error;
   PhoneEvent *e = (PhoneEvent *) g_malloc (sizeof (PhoneEvent));
@@ -402,16 +434,24 @@ static void ReadSMS (gpointer d, gpointer userData)
     {  
       gn_log_xdebug ("%d. %s   ", data->number, data->remote.number);
       gn_log_xdebug ("%02d-%02d-%02d %02d:%02d:%02d+%02d %s\n", data->smsc_time.year,
-                data->smsc_time.month, data->smsc_time.day, data->smsc_time.hour,
-                data->smsc_time.minute, data->smsc_time.second, data->smsc_time.timezone,
-                data->user_data[0].u.text);
+                     data->smsc_time.month, data->smsc_time.day, data->smsc_time.hour,
+                     data->smsc_time.minute, data->smsc_time.second, data->smsc_time.timezone,
+                     data->user_data[0].u.text);
       error = (*DB_InsertSMS) (data, smsdConfig.phone);
     }
     
     if (smsdConfig.logFile)
     {
       if (error)
-        LogFile (_("Inserting sms from %s unsuccessful.\n"), data->remote.number);
+      {
+        LogFile (_("Inserting sms from %s unsuccessful.\nDate: %02d-%02d-%02d %02d:%02d:%02d+%02d\nText: %s\n\nExiting."),
+                 data->remote.number, data->smsc_time.year,
+                 data->smsc_time.month, data->smsc_time.day,
+                 data->smsc_time.hour, data->smsc_time.minute,
+                 data->smsc_time.second, data->smsc_time.timezone,
+                 data->user_data[0].u.text);
+        MainExit (1);
+      }
       else
         LogFile (_("Inserting sms from %s successful.\n"), data->remote.number);
     }
@@ -443,25 +483,6 @@ static void GetSMS (void)
 }
 
 
-static void MainExit (gint sig)
-{
-  PhoneEvent *e = (PhoneEvent *) g_malloc (sizeof (PhoneEvent));
-
-  e->event = Event_Exit;
-  e->data = NULL;
-  InsertEvent (e);
-  
-  pthread_mutex_lock (&db_monitorMutex);
-  db_monitor = FALSE;
-  pthread_mutex_unlock (&db_monitorMutex);
-  
-  pthread_join (monitor_th, NULL);
-  pthread_join (db_monitor_th, NULL);
-  (*DB_Bye) ();
-  exit (0);
-}
-
-
 static void Run (void)
 {
   struct sigaction act;
@@ -493,7 +514,8 @@ static void Run (void)
 int main (int argc, char *argv[])
 {
 #ifdef ENABLE_NLS
-  textdomain("gnokii");
+  textdomain ("gnokii");
+  setlocale (LC_ALL, "");
 #endif
 
   gn_elog_handler = NULL;
