@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_logos.c,v 1.25 2003/02/26 00:15:53 pkot Exp $
+  $Id: xgnokii_logos.c,v 1.46 2007/06/02 10:39:44 dforsi Exp $
    
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999-2005 Jan Derfinak
+  Copyright (C) 2000-2001 Marcin Wiacek, Chris Kemp
+  Copyright (C) 2002-2006 Pawel Kot
+  Copyright (C) 2002      Markus Plail
 
 */
 
@@ -33,6 +36,8 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <string.h>
+#include <dirent.h>
+
 #include <gtk/gtk.h>
 
 #include "misc.h"
@@ -79,12 +84,15 @@ int drawingAreaWidth, drawingAreaHeight;	/* in pixels */
 int mouseButtonPushed = 0;
 
 /* stuff for previewArea */
+#define MAX_PIXMAPS 100
 GtkWidget *previewArea = NULL;
 GdkPixmap *previewPixmap = NULL;
 GdkPixmap *greenPixelPixmap;
 int previewPixmapWidth, previewPixmapHeight;
 int previewAvailable = 1, showPreviewErrorDialog = 1;
-int previewPixmapNumber = 0;
+int previewPixmapNumber = 0, pixmapFiles = 0;
+int pixmapsInitialized = FALSE, pixmapDefaultId = -1;
+gchar *pixmapNames[MAX_PIXMAPS];
 
 gn_bmp bitmap, oldBitmap;
 gn_network_info networkInfo;
@@ -143,94 +151,65 @@ int GetMaxFrom3(int a, int b, int c)
 	}
 }
 
-/* load preview pixmap from file */
-GdkPixmap *GetPreviewPixmap(GtkWidget * widget)
+/* Read all Preview* files from the given path and put them into pixmapNames
+ * array. We support no more than MAX_PIXMAP files. */
+static void GetPixmaps(gchar *path)
 {
-	GdkPixmap *pixmap;
+	DIR *dir;
+	struct dirent *de;
+
+	dir = opendir(path);
+	while (dir && (de = readdir(dir))) {
+		if (pixmapFiles == MAX_PIXMAPS)
+			break;
+		if (!strncmp(de->d_name, "Preview_", 8)) {
+			pixmapNames[pixmapFiles] = g_strdup(de->d_name);
+			if (!strcmp(de->d_name, "Preview_default.xpm"))
+				pixmapDefaultId = pixmapFiles;
+			pixmapFiles++;
+		}
+	}
+	pixmapsInitialized = TRUE;
+}
+
+/* Get the pixmap that filename contains model string. If there's none, get
+ * default one. */
+static int GetPixmapId(gchar *model)
+{
+	int i, pos = pixmapDefaultId;
+
+	for (i = 0; i < pixmapFiles; i++) {
+		if (strstr(pixmapNames[i], model)) {
+			pos = i;
+			break;
+		}
+	}
+	return pos;
+}
+
+/* Load preview pixmap from file. On the first run do the initialization.
+ * Next runs are after clicking on the preview area that should switch to the
+ * next pixmap. In case none is available NULL is returned. */
+GdkPixmap *GetPreviewPixmap(GtkWidget *widget)
+{
+	GdkPixmap *pixmap = NULL;
 	GdkBitmap *mask;
-	gchar *file;
+	gchar *file, *dirname;
 
-	if (previewPixmapNumber == 0) {
-		if (!strcmp(xgnokiiConfig.model, "6110") || !strcmp(xgnokiiConfig.model, "6120")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6110.xpm");
-			previewPixmapNumber = 1;
-		} else if (!strcmp(xgnokiiConfig.model, "6130") ||
-			   !strcmp(xgnokiiConfig.model, "6150") ||
-			   !strcmp(xgnokiiConfig.model, "616x") ||
-			   !strcmp(xgnokiiConfig.model, "6185") ||
-			   !strcmp(xgnokiiConfig.model, "6190")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6150.xpm");
-			previewPixmapNumber = 2;
-		} else if (!strcmp(xgnokiiConfig.model, "3210")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_3210.xpm");
-			previewPixmapNumber = 3;
-		} else if (!strcmp(xgnokiiConfig.model, "3310") ||
-			   !strcmp(xgnokiiConfig.model, "3330")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_3310.xpm");
-			previewPixmapNumber = 4;
-		} else if (!strcmp(xgnokiiConfig.model, "5110") ||
-			   !strcmp(xgnokiiConfig.model, "5130")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_5110.xpm");
-			previewPixmapNumber = 5;
-		} else if (!strcmp(xgnokiiConfig.model, "6250")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6250.xpm");
-			previewPixmapNumber = 6;
-		} else if (!strcmp(xgnokiiConfig.model, "7110")) {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_7110.xpm");
-			previewPixmapNumber = 7;
-		} else {
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6210.xpm");
-			previewPixmapNumber = 8;
-		}
-	} else
-		switch (previewPixmapNumber) {
-		case 1:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6110.xpm");
-			break;
-		case 2:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6150.xpm");
-			break;
-		case 3:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_3210.xpm");
-			break;
-		case 4:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_3310.xpm");
-			break;
-		case 5:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_5110.xpm");
-			break;
-		case 6:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6250.xpm");
-			break;
-		case 7:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_7110.xpm");
-			break;
-		default:
-			file = g_strdup_printf("%s%s", xgnokiiConfig.xgnokiidir,
-					       "/xpm/Preview_6210.xpm");
-			break;
-		}
+	dirname = g_strdup_printf("%s/xpm/", xgnokiiConfig.xgnokiidir);
+	if (!pixmapsInitialized) {
+		GetPixmaps(dirname);
+		previewPixmapNumber = GetPixmapId(xgnokiiConfig.model);
+	}
 
-	pixmap = gdk_pixmap_create_from_xpm(widget->window, &mask,
+	if (previewPixmapNumber >= 0) {
+		file = g_strdup_printf("%s%s", dirname,
+					pixmapNames[previewPixmapNumber]);
+		pixmap = gdk_pixmap_create_from_xpm(widget->window, &mask,
 					    &widget->style->bg[GTK_STATE_NORMAL], file);
-
-	g_free(file);
-
+		g_free(file);
+	}
+	g_free(dirname);
 	return pixmap;
 }
 
@@ -730,7 +709,7 @@ void UpdateToolScreen(GtkWidget * widget, int x1, int y1, int x2, int y2)
 
 gint PreviewAreaButtonPressEvent(GtkWidget * widget, GdkEventButton * event)
 {
-	previewPixmapNumber = (previewPixmapNumber % 8) + 1;
+	previewPixmapNumber = ((previewPixmapNumber + 1) % pixmapFiles);
 
 	gtk_drawing_area_size(GTK_DRAWING_AREA(previewArea),
 			      previewPixmapWidth, previewPixmapHeight);
@@ -968,6 +947,7 @@ gint DrawingAreaExposeEvent(GtkWidget * widget, GdkEventExpose * event)
 void GetNetworkInfoEvent(GtkWidget * widget)
 {
 	gn_error error;
+	gchar *netcou;
 	PhoneEvent *e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent));
 	D_NetworkInfo *data = (D_NetworkInfo *) g_malloc(sizeof(D_NetworkInfo));
 
@@ -986,15 +966,18 @@ void GetNetworkInfoEvent(GtkWidget * widget)
 
 	/* watch for errors */
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("Error getting network info\n(error=%d)"), error);
+		gchar *buf = g_strdup_printf(_("Error getting network info\n%s"), gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
+		/* after the error network_code is undefined but later code won't know */
+		networkInfo.network_code[0] = '\0';
 	}
 
 	/* set new operator name to combo */
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry),
-			   gn_network_name_get(networkInfo.network_code));
+	netcou = g_strdup_printf("%s (%s)", gn_network_name_get(networkInfo.network_code),
+					    gn_network2country(networkInfo.network_code));
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry), netcou);
 }
 
 void GetLogoEvent(GtkWidget * widget)
@@ -1003,10 +986,12 @@ void GetLogoEvent(GtkWidget * widget)
 	int i;
 	PhoneEvent *e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent));
 	D_Bitmap *data = (D_Bitmap *) g_malloc(sizeof(D_Bitmap));
-	char *operator = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry));
+	gchar *netcou = (gchar *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry));
+	gchar network[64], country[24];
 
 	/* prepare data for event */
-	strncpy(bitmap.netcode, gn_network_code_get(operator), 7);
+	sscanf(netcou, "%s (%[^)])", network, country);
+	strncpy(bitmap.netcode, gn_network_code_find(network, country), 7);
 	data->bitmap = &bitmap;
 	e->event = Event_GetBitmap;
 	e->data = data;
@@ -1027,7 +1012,7 @@ void GetLogoEvent(GtkWidget * widget)
 
 	/* watch for errors */
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("Error getting bitmap\n(error=%d)"), error);
+		gchar *buf = g_strdup_printf(_("Error getting bitmap\n%s"), gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
@@ -1042,11 +1027,13 @@ void SetLogoEvent(GtkWidget * widget)
 	gn_error error;
 	PhoneEvent *e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent));
 	D_Bitmap *data = (D_Bitmap *) g_malloc(sizeof(D_Bitmap));
-	char *operator = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry));
+	gchar *netcou = (gchar *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(networkCombo)->entry));
+	gchar network[64], country[24];
 	int i;
 
 	/* prepare data */
-	strncpy(bitmap.netcode, gn_network_code_get(operator), 7);
+	sscanf(netcou, "%s (%[^)])", network, country);
+	strncpy(bitmap.netcode, gn_network_code_find(network, country), 7);
 
 	if (bitmap.type == GN_BMP_CallerLogo) {
 		/* above condition must be there, because if you launch logos before
@@ -1075,7 +1062,7 @@ void SetLogoEvent(GtkWidget * widget)
 
 	/* watch for errors */
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("Error setting bitmap\n(error=%d)"), error);
+		gchar *buf = g_strdup_printf(_("Error setting bitmap\n%s"), gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
@@ -1260,19 +1247,19 @@ gint ToolTypeEvent(GtkWidget * widget)
 	return 0;
 }
 
-/* this is launched when logo type was change by buttons on toolbar */
+/* this is launched when logo type was changed by buttons on toolbar */
 gint LogoTypeEvent(GtkWidget * widget)
 {
 	int clear = 0;
-	g_print("LogoTypeEvent called!\n");
-	g_print("height: %i, width: %i\n", statemachine.driver.phone.startup_logo_height,
-		statemachine.driver.phone.startup_logo_width);
+	gn_log_xdebug("LogoTypeEvent called!\n");
+	gn_log_xdebug("width: %i, height: %i\n", statemachine->driver.phone.startup_logo_width,
+		statemachine->driver.phone.startup_logo_height);
 	/* is startupLogo? */
 	/* Resize and clear anyway - CK */
 	if (GTK_TOGGLE_BUTTON(buttonStartup)->active) {
 		/* look for old bitmap type, clean if another */
 		clear = 1;
-		gn_bmp_resize(&bitmap, GN_BMP_StartupLogo, &statemachine.driver.phone);
+		gn_bmp_resize(&bitmap, GN_BMP_StartupLogo, &statemachine->driver.phone);
 	}
 
 	/* has phone support for callerGroups? */
@@ -1281,7 +1268,7 @@ gint LogoTypeEvent(GtkWidget * widget)
 			/* previous was startup? clear and draw batteries, signal, ... */
 			/* Clear anyway for 7110...CK */
 			clear = 1;
-			gn_bmp_resize(&bitmap, GN_BMP_CallerLogo, &statemachine.driver.phone);
+			gn_bmp_resize(&bitmap, GN_BMP_CallerLogo, &statemachine->driver.phone);
 		}
 	}
 
@@ -1291,10 +1278,10 @@ gint LogoTypeEvent(GtkWidget * widget)
 		/* previous startup? clear and draw batteries, signal, ... */
 		/* Clear anyway for 7110..CK */
 		clear = 1;
-		if (!strncmp(statemachine.driver.phone.models, "6510", 4))
-			gn_bmp_resize(&bitmap, GN_BMP_NewOperatorLogo, &statemachine.driver.phone);
+		if (!strncmp(statemachine->driver.phone.models, "6510", 4))
+			gn_bmp_resize(&bitmap, GN_BMP_NewOperatorLogo, &statemachine->driver.phone);
 		else
-			gn_bmp_resize(&bitmap, GN_BMP_OperatorLogo, &statemachine.driver.phone);
+			gn_bmp_resize(&bitmap, GN_BMP_OperatorLogo, &statemachine->driver.phone);
 	}
 
 	/* must clear? */
@@ -1328,9 +1315,9 @@ void ExportLogoFileMain(gchar * name)
 
 	strncpy(tbitmap.netcode, gn_network_code_get(networkInfo.network_code), 7);
 
-	error = gn_file_bitmap_save(name, &tbitmap, &statemachine.driver.phone);
+	error = gn_file_bitmap_save(name, &tbitmap, &statemachine->driver.phone);
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("Error saving file\n(error=%d)"), error);
+		gchar *buf = g_strdup_printf(_("Error writing file\n%s"), gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
@@ -1347,18 +1334,18 @@ static void ExportFileSelected(GtkWidget * w, GtkFileSelection * fs)
 {
 	static YesNoDialog dialog = { NULL, NULL };
 	FILE *f;
-	gchar err[80];
+	gchar err[255];
 
-	exportDialogData.fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+	exportDialogData.fileName = (gchar *) gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
 	gtk_widget_hide(GTK_WIDGET(fs));
 
 	if ((f = fopen(exportDialogData.fileName, "r")) != NULL) {
 		fclose(f);
 		if (dialog.dialog == NULL) {
-			CreateYesNoDialog(&dialog, YesLogoFileExportDialog, CancelDialog,
+			CreateYesNoDialog(&dialog, (GtkSignalFunc) YesLogoFileExportDialog, (GtkSignalFunc) CancelDialog,
 					  GUI_LogosWindow);
 			gtk_window_set_title(GTK_WINDOW(dialog.dialog), _("Overwrite file?"));
-			g_snprintf(err, 80, _("File %s already exist.\nOverwrite?"),
+			g_snprintf(err, 255, _("File %s already exists.\nOverwrite?"),
 				   exportDialogData.fileName);
 			gtk_label_set_text(GTK_LABEL(dialog.text), err);
 		}
@@ -1375,11 +1362,11 @@ void ImportFileSelected(GtkWidget * w, GtkFileSelection * fs)
 	gchar *fileName;
 	FILE *f;
 
-	fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+	fileName = (gchar *) gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
 	gtk_widget_hide(GTK_WIDGET(fs));
 
 	if ((f = fopen(fileName, "r")) == NULL) {
-		gchar *buf = g_strdup_printf(_("Can't open file %s for reading !"), fileName);
+		gchar *buf = g_strdup_printf(_("Can't open file %s for reading!\n"), fileName);
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
@@ -1387,9 +1374,9 @@ void ImportFileSelected(GtkWidget * w, GtkFileSelection * fs)
 	} else
 		fclose(f);
 
-	error = gn_file_bitmap_read(fileName, &tbitmap, &statemachine.driver.phone);
+	error = gn_file_bitmap_read(fileName, &tbitmap, &statemachine->driver.phone);
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("Error reading file\n(error=%d)"), error);
+		gchar *buf = g_strdup_printf(_("Error reading file\n%s"), gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
@@ -1481,27 +1468,27 @@ static GtkItemFactoryEntry logosMenuItems[] = {
 
 void InitLogosMenu(void)
 {
-	logosMenuItems[0].path = g_strdup(_("/_File"));
-	logosMenuItems[1].path = g_strdup(_("/File/_Open"));
-	logosMenuItems[2].path = g_strdup(_("/File/_Save"));
-	logosMenuItems[3].path = g_strdup(_("/File/Save _as ..."));
-	logosMenuItems[4].path = g_strdup(_("/File/Sep1"));
-	logosMenuItems[5].path = g_strdup(_("/File/_Get operator"));
-	logosMenuItems[6].path = g_strdup(_("/File/Get _logo"));
-	logosMenuItems[7].path = g_strdup(_("/File/Se_t logo"));
-	logosMenuItems[8].path = g_strdup(_("/File/Sep2"));
-	logosMenuItems[9].path = g_strdup(_("/File/_Close"));
-	logosMenuItems[10].path = g_strdup(_("/_Edit"));
-	logosMenuItems[11].path = g_strdup(_("/Edit/_Clear"));
-	logosMenuItems[12].path = g_strdup(_("/Edit/_Invert"));
-	logosMenuItems[13].path = g_strdup(_("/Edit/Sep3"));
-	logosMenuItems[14].path = g_strdup(_("/Edit/_Up logo"));
-	logosMenuItems[15].path = g_strdup(_("/Edit/_Down logo"));
-	logosMenuItems[16].path = g_strdup(_("/Edit/_Left logo"));
-	logosMenuItems[17].path = g_strdup(_("/Edit/_Right logo"));
-	logosMenuItems[18].path = g_strdup(_("/Edit/Sep4"));
-	logosMenuItems[19].path = g_strdup(_("/Edit/Flip _horizontal"));
-	logosMenuItems[20].path = g_strdup(_("/Edit/Flip _vertical"));
+	logosMenuItems[0].path = _("/_File");
+	logosMenuItems[1].path = _("/File/_Open");
+	logosMenuItems[2].path = _("/File/_Save");
+	logosMenuItems[3].path = _("/File/Save _as ...");
+	logosMenuItems[4].path = _("/File/Sep1");
+	logosMenuItems[5].path = _("/File/_Get operator");
+	logosMenuItems[6].path = _("/File/Get _logo");
+	logosMenuItems[7].path = _("/File/Se_t logo");
+	logosMenuItems[8].path = _("/File/Sep2");
+	logosMenuItems[9].path = _("/File/_Close");
+	logosMenuItems[10].path = _("/_Edit");
+	logosMenuItems[11].path = _("/Edit/_Clear");
+	logosMenuItems[12].path = _("/Edit/_Invert");
+	logosMenuItems[13].path = _("/Edit/Sep3");
+	logosMenuItems[14].path = _("/Edit/_Up logo");
+	logosMenuItems[15].path = _("/Edit/_Down logo");
+	logosMenuItems[16].path = _("/Edit/_Left logo");
+	logosMenuItems[17].path = _("/Edit/_Right logo");
+	logosMenuItems[18].path = _("/Edit/Sep4");
+	logosMenuItems[19].path = _("/Edit/Flip _horizontal");
+	logosMenuItems[20].path = _("/Edit/Flip _vertical");
 }
 
 void GUI_CreateLogosWindow(void)
@@ -1541,8 +1528,8 @@ void GUI_CreateLogosWindow(void)
 	CreateInfoDialog(&infoDialog, GUI_LogosWindow);
 
 	accelGroup = gtk_accel_group_new();
-	gtk_accel_group_attach(accelGroup, GTK_OBJECT(GUI_LogosWindow));
-
+	gtk_window_add_accel_group(GTK_WINDOW(GUI_LogosWindow), accelGroup);
+	
 /* create main vbox */
 	vbox = gtk_vbox_new(FALSE, 1);
 	gtk_container_add(GTK_CONTAINER(GUI_LogosWindow), vbox);
@@ -1556,9 +1543,9 @@ void GUI_CreateLogosWindow(void)
 	gtk_widget_show(menuBar);
 
 /* toolbar */
-	toolBar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolBar), GTK_RELIEF_NORMAL);
+	toolBar = gtk_toolbar_new();
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolBar), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolBar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolBar), NULL, _("Clear logo"), NULL,
 				NewPixmap(New_xpm, GUI_LogosWindow->window,
@@ -1623,13 +1610,18 @@ void GUI_CreateLogosWindow(void)
 	networkCombo = gtk_combo_new();
 	gtk_combo_set_use_arrows_always(GTK_COMBO(networkCombo), 1);
 
-	while (gn_network_get(&network, i++))
+	while (gn_network_get(&network, i++)) {
+		gchar *netcou;
+
+		netcou = g_strdup_printf("%s (%s)", network.name, gn_network2country(network.code));
 		glistNetwork = g_list_insert_sorted(glistNetwork,
-						    network.name, (GCompareFunc) strcmp);
+						    netcou, (GCompareFunc) strcmp);
+	}
 	gtk_combo_set_popdown_strings(GTK_COMBO(networkCombo), glistNetwork);
 	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(networkCombo)->entry), FALSE);
 	gtk_toolbar_append_widget(GTK_TOOLBAR(toolBar), networkCombo, "", "");
 	gtk_widget_show(networkCombo);
+	g_list_foreach(glistNetwork, (GFunc) g_free, NULL);
 	g_list_free(glistNetwork);
 
 	callerCombo = gtk_combo_new();
@@ -1727,9 +1719,9 @@ void GUI_CreateLogosWindow(void)
 	gtk_widget_show(drawingArea);
 
 	/* vertical tool bar */
-	vertToolBar = gtk_toolbar_new(GTK_ORIENTATION_VERTICAL, GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(vertToolBar), GTK_RELIEF_NORMAL);
+	vertToolBar = gtk_toolbar_new();
 	gtk_toolbar_set_style(GTK_TOOLBAR(vertToolBar), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(vertToolBar), GTK_ORIENTATION_VERTICAL);
 
 	buttonBrush = gtk_toolbar_append_element(GTK_TOOLBAR(vertToolBar),
 						 GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL, NULL,
@@ -1834,8 +1826,8 @@ void GUI_ShowLogosWindow(void)
 
 	/* Call to reset Startup logo size */
 	LogoTypeEvent(GUI_LogosWindow);
-	g_print("width: %i, height: %i\n", bitmap.width, bitmap.height);
-	gtk_widget_show(GUI_LogosWindow);
+	gn_log_xdebug("width: %i, height: %i\n", bitmap.width, bitmap.height);
+	gtk_window_present(GTK_WINDOW(GUI_LogosWindow));
 
 	if (!previewAvailable && showPreviewErrorDialog) {
 		gchar *buf = g_strdup(_("Load preview pixmap error, feature disabled."));
