@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_speed.c,v 1.22 2006/08/15 10:57:20 dforsi Exp $
+  $Id: xgnokii_speed.c,v 1.24 2007/05/31 12:09:47 dforsi Exp $
 
   X G N O K I I
 
@@ -195,13 +195,13 @@ static void ReadSpeedDial(void)
 
 	for (i = 2; i < 10; i++) {
 		if ((d = (D_SpeedDial *) g_malloc(sizeof(D_SpeedDial))) == NULL) {
-			g_print(_("Cannot allocate memory!\n"));
+			g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__, __LINE__);
 			return;
 		}
 		memset(d, 0, sizeof(D_SpeedDial));
 		d->entry.number = i;
 		if ((e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent))) == NULL) {
-			g_print(_("Cannot allocate memory!\n"));
+			g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__, __LINE__);
 			g_free(d);
 			return;
 		}
@@ -261,7 +261,7 @@ static void SaveSpeedDial(void)
 				if (d->entry.location == 0)
 					continue;
 				if ((e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent))) == NULL) {
-					g_print(_("Cannot allocate memory!\n"));
+					g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__, __LINE__);
 					return;
 				}
 
@@ -345,7 +345,7 @@ static void OkImportDialog(GtkWidget * w, GtkFileSelection * fs)
 	i = 0;
 	while (fgets(buf, IO_BUF_LEN, f) && i++ < 9) {
 		if ((d = (D_SpeedDial *) g_malloc(sizeof(D_SpeedDial))) == NULL) {
-			g_print(_("Cannot allocate memory!\n"));
+			g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__, __LINE__);
 			gtk_clist_clear(GTK_CLIST(clist));
 			gtk_clist_sort(GTK_CLIST(clist));
 			gtk_clist_thaw(GTK_CLIST(clist));
@@ -522,19 +522,19 @@ static GtkItemFactoryEntry menu_items[] = {
 
 static void InitMainMenu(void)
 {
-	menu_items[0].path = g_strdup(_("/_File"));
-	menu_items[1].path = g_strdup(_("/File/_Read from phone"));
-	menu_items[2].path = g_strdup(_("/File/_Save to phone"));
-	menu_items[3].path = g_strdup(_("/File/Sep1"));
-	menu_items[4].path = g_strdup(_("/File/_Import from file"));
-	menu_items[5].path = g_strdup(_("/File/_Export to file"));
-	menu_items[6].path = g_strdup(_("/File/Sep2"));
-	menu_items[7].path = g_strdup(_("/File/_Close"));
-	menu_items[8].path = g_strdup(_("/_Edit"));
-	menu_items[9].path = g_strdup(_("/Edit/_Edit"));
-	menu_items[10].path = g_strdup(_("/_Help"));
-	menu_items[11].path = g_strdup(_("/Help/_Help"));
-	menu_items[12].path = g_strdup(_("/Help/_About"));
+	menu_items[0].path = _("/_File");
+	menu_items[1].path = _("/File/_Read from phone");
+	menu_items[2].path = _("/File/_Save to phone");
+	menu_items[3].path = _("/File/Sep1");
+	menu_items[4].path = _("/File/_Import from file");
+	menu_items[5].path = _("/File/_Export to file");
+	menu_items[6].path = _("/File/Sep2");
+	menu_items[7].path = _("/File/_Close");
+	menu_items[8].path = _("/_Edit");
+	menu_items[9].path = _("/Edit/_Edit");
+	menu_items[10].path = _("/_Help");
+	menu_items[11].path = _("/Help/_Help");
+	menu_items[12].path = _("/Help/_About");
 }
 
 
@@ -631,8 +631,7 @@ void GUI_CreateSpeedDialWindow(void)
 
 	for (i = 0; i < 3; i++) {
 		if ((sColumn = g_malloc(sizeof(SortColumn))) == NULL) {
-			g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__,
-				__LINE__);
+			g_print(_("Error: %s: line %d: Can't allocate memory!\n"), __FILE__, __LINE__);
 			gtk_main_quit();
 		}
 		sColumn->clist = clist;
